@@ -103,8 +103,8 @@
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 /* Python module definition of a SAX html parser */
-#include "htmlsax.h"
-#include "structmember.h"
+#include "htmlsax.h"      /* SAX interface (includes Python.h) */
+#include "structmember.h" /* Python include for object definition */
 #include <string.h>
 #include <stdio.h>
 
@@ -129,6 +129,8 @@ static int yyerror (char* msg) {
     return 0;
 }
 
+/* existing Python methods */
+
 /* parser.resolve_entities */
 static PyObject* resolve_entities;
 /* ListDict class, sorted dictionary */
@@ -154,21 +156,29 @@ static PyObject* u_meta;
     if (b == NULL) { Py_DECREF(self); return NULL; } \
     (b)[0] = '\0'
 
-#define CHECK_ERROR(ud, label) \
-    if (ud->error && PyObject_HasAttrString(ud->handler, "error") == 1) { \
-	callback = PyObject_GetAttrString(ud->handler, "error"); \
-	if (!callback) { error = 1; goto label; } \
-	result = PyObject_CallFunction(callback, "O", ud->error); \
-	if (!result) { error = 1; goto label; } \
+/* check an error condition and if true set error flag and goto given label */
+#define CHECK_ERROR(cond, label) \
+    if (cond) { \
+        error = 1; \
+        goto label; \
     }
 
-/* generic callback macro */
+/* check the parser error flag and if set goto given label */
+#define CHECK_PARSER_ERROR(ud, label) \
+    if (ud->error && PyObject_HasAttrString(ud->handler, "error") == 1) { \
+	callback = PyObject_GetAttrString(ud->handler, "error"); \
+	CHECK_ERROR((!callback), label); \
+	result = PyObject_CallFunction(callback, "O", ud->error); \
+	CHECK_ERROR((!result), label); \
+    }
+
+/* generic Python callback macro */
 #define CALLBACK(ud, attr, format, arg, label) \
     if (PyObject_HasAttrString(ud->handler, attr) == 1) { \
 	callback = PyObject_GetAttrString(ud->handler, attr); \
-	if (callback == NULL) { error = 1; goto label; } \
+	CHECK_ERROR((callback == NULL), label); \
 	result = PyObject_CallFunction(callback, format, arg); \
-	if (result == NULL) { error = 1; goto label; } \
+	CHECK_ERROR((result == NULL), label); \
 	Py_CLEAR(callback); \
 	Py_CLEAR(result); \
     }
@@ -197,18 +207,24 @@ staticforward PyTypeObject parser_type;
 #define YYMALLOC PyMem_Malloc
 #define YYFREE PyMem_Free
 
-/* Test whether tag does not need an HTML end tag. Return -1 on error. */
+/* Test whether tag does not need an HTML end tag.
+   @ptag: ASCII encoded Python string in lowercase (!)
+   @parser: SAX parser object
+   @return: < 0 on error, > 0 if HTML end tag is needed, else 0
+*/
 static int html_end_tag (PyObject* ptag, PyObject* parser) {
-    PyObject* pdoctype;
+    PyObject* pdoctype = NULL;
     char* doctype;
+    int error = 0;
     int ret = 1;
     pdoctype = PyObject_GetAttrString(parser, "doctype");
-    if (pdoctype == NULL) return -1;
+    CHECK_ERROR((pdoctype == NULL), finish_html_end_tag);
     doctype = PyString_AsString(pdoctype);
-    if (doctype == NULL) { Py_DECREF(pdoctype); return -1; }
+    CHECK_ERROR((doctype == NULL), finish_html_end_tag);
+    /* check for HTML (else it's presumably XHTML) */
     if (strcmp(doctype, "HTML") == 0) {
         char* tag = PyString_AsString(ptag);
-        if (tag == NULL) { Py_DECREF(pdoctype); return -1; }
+        CHECK_ERROR((tag == NULL), finish_html_end_tag);
         ret = strcmp(tag, "area")!=0 &&
             strcmp(tag, "base")!=0 &&
             strcmp(tag, "basefont")!=0 &&
@@ -223,8 +239,11 @@ static int html_end_tag (PyObject* ptag, PyObject* parser) {
             strcmp(tag, "meta")!=0 &&
             strcmp(tag, "param")!=0;
     }
-    /* it is not HTML (presumably XHTML) */
-    Py_DECREF(pdoctype);
+finish_html_end_tag:
+    Py_XDECREF(pdoctype);
+    if (error) {
+        return -1;
+    }
     return ret;
 }
 
@@ -256,7 +275,7 @@ typedef int YYSTYPE;
 
 
 /* Line 214 of yacc.c.  */
-#line 260 "htmlparse.c"
+#line 279 "htmlparse.c"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
@@ -433,8 +452,8 @@ static const yysigned_char yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned short int yyrline[] =
 {
-       0,   170,   170,   171,   174,   175,   182,   225,   276,   309,
-     330,   351,   372,   397,   422,   447
+       0,   189,   189,   192,   197,   201,   208,   252,   303,   342,
+     364,   385,   407,   433,   460,   487
 };
 #endif
 
@@ -1140,22 +1159,29 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 170 "htmlparse.y"
-    {;}
+#line 189 "htmlparse.y"
+    {
+    /* parse a single element */
+;}
     break;
 
   case 3:
-#line 171 "htmlparse.y"
-    {;}
+#line 192 "htmlparse.y"
+    {
+    /* parse a list of elements */
+;}
     break;
 
   case 4:
-#line 174 "htmlparse.y"
-    { YYACCEPT; /* wait for more lexer input */ ;}
+#line 197 "htmlparse.y"
+    {
+    /* wait for more lexer input */
+    YYACCEPT;
+;}
     break;
 
   case 5:
-#line 176 "htmlparse.y"
+#line 202 "htmlparse.y"
     {
     /* an error occured in the scanner, the python exception must be set */
     UserData* ud = yyget_extra(scanner);
@@ -1165,10 +1191,11 @@ yyreduce:
     break;
 
   case 6:
-#line 183 "htmlparse.y"
+#line 209 "htmlparse.y"
     {
-    /* $1 is a PyTuple (<tag>, <attrs>)
-       <tag> is a PyObject, <attrs> is a PyDict */
+    /* parsed HTML start tag (eg. <a href="blubb">)
+       $1 is a PyTuple (<tag>, <attrs>)
+       <tag> is a PyObject, <attrs> is a ListDict */
     UserData* ud = yyget_extra(scanner);
     PyObject* callback = NULL;
     PyObject* result = NULL;
@@ -1176,24 +1203,24 @@ yyreduce:
     PyObject* attrs = PyTuple_GET_ITEM(yyvsp[0], 1);
     int error = 0;
     int cmp;
-    if (tag == NULL || attrs == NULL) { error = 1; goto finish_start; }
+    CHECK_ERROR((tag == NULL || attrs == NULL), finish_start);
     cmp = PyObject_RichCompareBool(tag, u_meta, Py_EQ);
-    if (cmp == -1) { error = 1; goto finish_start; }
+    CHECK_ERROR((cmp == -1), finish_start);
     if (cmp == 1) {
         /* set encoding */
         result = PyObject_CallFunction(set_encoding, "OO", ud->parser, attrs);
-        if (result == NULL) { error = 1; goto finish_start; }
+        CHECK_ERROR((result == NULL), finish_start);
         Py_CLEAR(result);
     }
     if (PyObject_HasAttrString(ud->handler, "start_element") == 1) {
 	callback = PyObject_GetAttrString(ud->handler, "start_element");
-	if (!callback) { error = 1; goto finish_start; }
+	CHECK_ERROR((!callback), finish_start);
 	result = PyObject_CallFunction(callback, "OO", tag, attrs);
-	if (!result) { error = 1; goto finish_start; }
+	CHECK_ERROR((!result), finish_start);
 	Py_CLEAR(callback);
         Py_CLEAR(result);
     }
-    CHECK_ERROR(ud, finish_start);
+    CHECK_PARSER_ERROR(ud, finish_start);
 finish_start:
     Py_XDECREF(ud->error);
     ud->error = NULL;
@@ -1211,10 +1238,11 @@ finish_start:
     break;
 
   case 7:
-#line 226 "htmlparse.y"
+#line 253 "htmlparse.y"
     {
-    /* $1 is a PyTuple (<tag>, <attrs>)
-       <tag> is a PyObject, <attrs> is a PyDict */
+    /* parsed HTML start-end tag (eg. <br/>)
+       $1 is a PyTuple (<tag>, <attrs>)
+       <tag> is a PyObject, <attrs> is a ListDict */
     UserData* ud = yyget_extra(scanner);
     PyObject* callback = NULL;
     PyObject* result = NULL;
@@ -1224,30 +1252,29 @@ finish_start:
     int cmp;
     char* fname;
     PyObject* tagname;
-    if (tag == NULL || attrs == NULL) { error = 1; goto finish_start_end; }
+    CHECK_ERROR((tag == NULL || attrs == NULL), finish_start_end);
     tagname = PyUnicode_AsEncodedString(tag, "ascii", "ignore");
-    if (tagname == NULL) { error = 1; goto finish_start_end; }
+    CHECK_ERROR((tagname == NULL), finish_start_end);
     cmp = PyObject_RichCompareBool(tag, u_meta, Py_EQ);
-    if (cmp == -1) { error = 1; goto finish_start; }
+    CHECK_ERROR((cmp == -1), finish_start_end);
     if (cmp == 1) {
         /* set encoding */
         result = PyObject_CallFunction(set_encoding, "OO", ud->parser, attrs);
-        if (result == NULL) { error = 1; goto finish_start_end; }
+        CHECK_ERROR((result == NULL), finish_start_end);
         Py_CLEAR(result);
     }
-    if (html_end_tag(tagname, ud->parser))
-        fname = "start_end_element";
-    else
-        fname = "start_element";
+    cmp = html_end_tag(tagname, ud->parser);
+    CHECK_ERROR((cmp < 0), finish_start_end);
+    fname = (cmp == 0 ? "start_element" : "start_end_element");
     if (PyObject_HasAttrString(ud->handler, fname) == 1) {
 	callback = PyObject_GetAttrString(ud->handler, fname);
-	if (!callback) { error = 1; goto finish_start_end; }
+	CHECK_ERROR((!callback), finish_start_end);
 	result = PyObject_CallFunction(callback, "OO", tag, attrs);
-	if (!result) { error = 1; goto finish_start_end; }
+	CHECK_ERROR((!result), finish_start_end);
 	Py_CLEAR(callback);
         Py_CLEAR(result);
     }
-    CHECK_ERROR(ud, finish_start_end);
+    CHECK_PARSER_ERROR(ud, finish_start_end);
 finish_start_end:
     Py_XDECREF(ud->error);
     ud->error = NULL;
@@ -1265,26 +1292,32 @@ finish_start_end:
     break;
 
   case 8:
-#line 277 "htmlparse.y"
+#line 304 "htmlparse.y"
     {
-    /* $1 is a PyUnicode */
+    /* parsed HTML end tag (eg. </b>)
+       $1 is a PyUnicode with the tag name */
     UserData* ud = yyget_extra(scanner);
     PyObject* callback = NULL;
     PyObject* result = NULL;
     int error = 0;
+    int cmp;
     /* encode tagname in ASCII, ignoring any unknown chars */
     PyObject* tagname = PyUnicode_AsEncodedString(yyvsp[0], "ascii", "ignore");
-    if (tagname == NULL) { error = 1; goto finish_end; }
-    if (PyObject_HasAttrString(ud->handler, "end_element") == 1 &&
-	html_end_tag(tagname, ud->parser)) {
+    if (tagname == NULL) {
+        error = 1;
+        goto finish_end;
+    }
+    cmp = html_end_tag(tagname, ud->parser);
+    CHECK_ERROR((cmp < 0), finish_end);
+    if (PyObject_HasAttrString(ud->handler, "end_element") == 1 && cmp > 0) {
 	callback = PyObject_GetAttrString(ud->handler, "end_element");
-	if (callback == NULL) { error = 1; goto finish_end; }
+	CHECK_ERROR((callback == NULL), finish_end);
 	result = PyObject_CallFunction(callback, "O", yyvsp[0]);
-	if (result == NULL) { error = 1; goto finish_end; }
+	CHECK_ERROR((result == NULL), finish_end);
 	Py_CLEAR(callback);
 	Py_CLEAR(result);
     }
-    CHECK_ERROR(ud, finish_end);
+    CHECK_PARSER_ERROR(ud, finish_end);
 finish_end:
     Py_XDECREF(ud->error);
     ud->error = NULL;
@@ -1301,15 +1334,16 @@ finish_end:
     break;
 
   case 9:
-#line 310 "htmlparse.y"
+#line 343 "htmlparse.y"
     {
-    /* $1 is a PyUnicode */
+    /* parsed HTML comment (eg. <!-- bla -->)
+       $1 is a PyUnicode with the comment content */
     UserData* ud = yyget_extra(scanner);
     PyObject* callback = NULL;
     PyObject* result = NULL;
     int error = 0;
     CALLBACK(ud, "comment", "O", yyvsp[0], finish_comment);
-    CHECK_ERROR(ud, finish_comment);
+    CHECK_PARSER_ERROR(ud, finish_comment);
 finish_comment:
     Py_XDECREF(ud->error);
     ud->error = NULL;
@@ -1325,7 +1359,7 @@ finish_comment:
     break;
 
   case 10:
-#line 331 "htmlparse.y"
+#line 365 "htmlparse.y"
     {
     /* $1 is a PyUnicode */
     UserData* ud = yyget_extra(scanner);
@@ -1333,7 +1367,7 @@ finish_comment:
     PyObject* result = NULL;
     int error = 0;
     CALLBACK(ud, "pi", "O", yyvsp[0], finish_pi);
-    CHECK_ERROR(ud, finish_pi);
+    CHECK_PARSER_ERROR(ud, finish_pi);
 finish_pi:
     Py_XDECREF(ud->error);
     ud->error = NULL;
@@ -1349,15 +1383,16 @@ finish_pi:
     break;
 
   case 11:
-#line 352 "htmlparse.y"
+#line 386 "htmlparse.y"
     {
-    /* $1 is a PyUnicode */
+    /* parsed HTML CDATA (eg. <![CDATA[spam and eggs ...]]>)
+       $1 is a PyUnicode with the CDATA content */
     UserData* ud = yyget_extra(scanner);
     PyObject* callback = NULL;
     PyObject* result = NULL;
     int error = 0;
     CALLBACK(ud, "cdata", "O", yyvsp[0], finish_cdata);
-    CHECK_ERROR(ud, finish_cdata);
+    CHECK_PARSER_ERROR(ud, finish_cdata);
 finish_cdata:
     Py_XDECREF(ud->error);
     ud->error = NULL;
@@ -1373,19 +1408,20 @@ finish_cdata:
     break;
 
   case 12:
-#line 373 "htmlparse.y"
+#line 408 "htmlparse.y"
     {
-    /* $1 is a PyUnicode */
+    /* parsed HTML doctype (eg. <!DOCTYPE imadoofus system>)
+       $1 is a PyUnicode with the doctype content */
     UserData* ud = yyget_extra(scanner);
     PyObject* callback = NULL;
     PyObject* result = NULL;
     int error = 0;
     /* set encoding */
     result = PyObject_CallFunction(set_doctype, "OO", ud->parser, yyvsp[0]);
-    if (result == NULL) { error = 1; goto finish_doctype; }
+    CHECK_ERROR((result == NULL), finish_doctype);
     Py_CLEAR(result);
     CALLBACK(ud, "doctype", "O", yyvsp[0], finish_doctype);
-    CHECK_ERROR(ud, finish_doctype);
+    CHECK_PARSER_ERROR(ud, finish_doctype);
 finish_doctype:
     Py_XDECREF(ud->error);
     ud->error = NULL;
@@ -1401,18 +1437,20 @@ finish_doctype:
     break;
 
   case 13:
-#line 398 "htmlparse.y"
+#line 434 "htmlparse.y"
     {
-    /* $1 is a PyUnicode */
+    /* parsed HTML script content (plus end tag which is omitted)
+       $1 is a PyUnicode with the script content */
     UserData* ud = yyget_extra(scanner);
     PyObject* callback = NULL;
     PyObject* result = NULL;
     int error = 0;
     PyObject* script = PyUnicode_DecodeASCII("script", 6, "ignore");
-    if (script == NULL) { error = 1; goto finish_script; }
+    CHECK_ERROR((script == NULL), finish_script);
     CALLBACK(ud, "characters", "O", yyvsp[0], finish_script);
+    /* emit the omitted end tag */
     CALLBACK(ud, "end_element", "O", script, finish_script);
-    CHECK_ERROR(ud, finish_script);
+    CHECK_PARSER_ERROR(ud, finish_script);
 finish_script:
     Py_XDECREF(ud->error);
     ud->error = NULL;
@@ -1429,18 +1467,20 @@ finish_script:
     break;
 
   case 14:
-#line 423 "htmlparse.y"
+#line 461 "htmlparse.y"
     {
-    /* $1 is a PyUnicode */
+    /* parsed HTML style content (plus end tag which is omitted)
+       $1 is a PyUnicode with the style content */
     UserData* ud = yyget_extra(scanner);
     PyObject* callback = NULL;
     PyObject* result = NULL;
     int error = 0;
     PyObject* style = PyUnicode_DecodeASCII("style", 5, "ignore");
-    if (style == NULL) { error = 1; goto finish_style; }
+    CHECK_ERROR((style == NULL), finish_style);
     CALLBACK(ud, "characters", "O", yyvsp[0], finish_style);
+    /* emit the omitted end tag */
     CALLBACK(ud, "end_element", "O", style, finish_style);
-    CHECK_ERROR(ud, finish_style);
+    CHECK_PARSER_ERROR(ud, finish_style);
 finish_style:
     Py_XDECREF(ud->error);
     ud->error = NULL;
@@ -1457,16 +1497,18 @@ finish_style:
     break;
 
   case 15:
-#line 448 "htmlparse.y"
+#line 488 "htmlparse.y"
     {
-    /* $1 is a PyUnicode */
-    /* Remember this is also called as a lexer error fallback */
+    /* parsed HTML text data
+       $1 is a PyUnicode with the text */
+    /* Remember this is also called as a lexer fallback when no
+       HTML structure element could be recognized. */
     UserData* ud = yyget_extra(scanner);
     PyObject* callback = NULL;
     PyObject* result = NULL;
     int error = 0;
     CALLBACK(ud, "characters", "O", yyvsp[0], finish_characters);
-    CHECK_ERROR(ud, finish_characters);
+    CHECK_PARSER_ERROR(ud, finish_characters);
 finish_characters:
     Py_XDECREF(ud->error);
     ud->error = NULL;
@@ -1485,7 +1527,7 @@ finish_characters:
     }
 
 /* Line 1010 of yacc.c.  */
-#line 1489 "htmlparse.c"
+#line 1531 "htmlparse.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1710,7 +1752,7 @@ yyreturn:
 }
 
 
-#line 471 "htmlparse.y"
+#line 513 "htmlparse.y"
 
 
 /* create parser object */
@@ -2005,12 +2047,14 @@ static PyObject* parser_debug (parser_object* self, PyObject* args) {
 }
 
 
+/* get SAX handler object */
 static PyObject* parser_gethandler (parser_object* self, void* closure) {
     Py_INCREF(self->handler);
     return self->handler;
 }
 
 
+/* set SAX handler object */
 static int parser_sethandler (parser_object* self, PyObject* value, void* closure) {
     if (value == NULL) {
        PyErr_SetString(PyExc_TypeError, "Cannot delete parser handler");
@@ -2024,12 +2068,14 @@ static int parser_sethandler (parser_object* self, PyObject* value, void* closur
 }
 
 
+/* get parser encoding */
 static PyObject* parser_getencoding (parser_object* self, void* closure) {
     Py_INCREF(self->encoding);
     return self->encoding;
 }
 
 
+/* set parser encoding */
 static int parser_setencoding (parser_object* self, PyObject* value, void* closure) {
     if (value == NULL) {
         PyErr_SetString(PyExc_TypeError, "Cannot delete encoding");
@@ -2046,12 +2092,14 @@ static int parser_setencoding (parser_object* self, PyObject* value, void* closu
 }
 
 
+/* get parser doctype */
 static PyObject* parser_getdoctype (parser_object* self, void* closure) {
     Py_INCREF(self->doctype);
     return self->doctype;
 }
 
 
+/* set parser doctype */
 static int parser_setdoctype (parser_object* self, PyObject* value, void* closure) {
     if (value == NULL) {
         PyErr_SetString(PyExc_TypeError, "Cannot delete doctype");
@@ -2151,25 +2199,6 @@ static PyTypeObject parser_type = {
 };
 
 
-/* python module interface 
-     "Create a new HTML parser object with handler (which may be None).\n"
-     "\n"
-     "Used callbacks (they don't have to be defined) of a handler are:\n"
-     "comment(data): <!--data-->\n"
-     "start_element(tag, attrs): <tag {attr1:value1,attr2:value2,..}>\n"
-     "end_element(tag): </tag>\n"
-     "doctype(data): <!DOCTYPE data?>\n"
-     "pi(name, data=None): <?name data?>\n"
-     "cdata(data): <![CDATA[data]]>\n"
-     "characters(data): data\n"
-     "\n"
-     "Additionally, there are error and warning callbacks:\n"
-     "error(msg)\n"
-     "warning(msg)\n"
-     "fatal_error(msg)\n"},
-
-*/
-
 static PyMethodDef htmlsax_methods[] = {
     {NULL} /* Sentinel */
 };
