@@ -33,6 +33,7 @@ import _linkchecker_configdata as configdata
 LOG = "linkcheck"
 LOG_CMDLINE = "linkcheck.cmdline"
 LOG_CHECK = "linkcheck.check"
+LOG_DNS = "linkcheck.dns"
 LOG_CACHE = "linkcheck.cache"
 LOG_GUI = "linkcheck.gui"
 LOG_THREAD = "linkcheck.thread"
@@ -41,6 +42,7 @@ lognames = {
     "checking": LOG_CHECK,
     "cache": LOG_CACHE,
     "gui": LOG_GUI,
+    "dns": LOG_DNS,
     "thread": LOG_THREAD,
     "all": LOG,
     }
@@ -69,7 +71,7 @@ def get_link_pat (arg, strict=False):
     @rtype: dict
     """
     linkcheck.log.debug(LOG_CHECK, "Link pattern %r", arg)
-    if arg[0:1] == '!':
+    if arg.startswith('!'):
         pattern = arg[1:]
         negate = True
     else:
@@ -90,7 +92,8 @@ import linkcheck.logger.dot
 import linkcheck.logger.sql
 import linkcheck.logger.csvlog
 import linkcheck.logger.blacklist
-import linkcheck.logger.xmllog
+import linkcheck.logger.gxml
+import linkcheck.logger.customxml
 import linkcheck.logger.none
 
 
@@ -103,7 +106,8 @@ Loggers = {
     "sql": linkcheck.logger.sql.SQLLogger,
     "csv": linkcheck.logger.csvlog.CSVLogger,
     "blacklist": linkcheck.logger.blacklist.BlacklistLogger,
-    "xml": linkcheck.logger.xmllog.XMLLogger,
+    "gxml": linkcheck.logger.gxml.GraphXMLLogger,
+    "xml": linkcheck.logger.customxml.CustomXMLLogger,
     "none": linkcheck.logger.none.NoneLogger,
 }
 # for easy printing: a comma separated logger list
