@@ -24,28 +24,24 @@ import urllib
 import cStringIO as StringIO
 
 import linkcheck
-import urlbase
 import proxysupport
 import httpurl
+import internpaturl
 import linkcheck.ftpparse._ftpparse as ftpparse
 
 DEFAULT_TIMEOUT_SECS = 300
 
 
-class FtpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
+class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
     """
     Url link with ftp scheme.
     """
 
-    def __init__ (self, base_url, recursion_level, consumer,
-                  parent_url = None,
-                  base_ref = None, line=0, column=0, name=u""):
+    def reset (self):
         """
         Initialize FTP url data.
         """
-        super(FtpUrl, self).__init__(base_url, recursion_level, consumer,
-             parent_url=parent_url, base_ref=base_ref,
-             line=line, column=column, name=name)
+        super(FtpUrl, self).reset()
         # list of files for recursion
         self.files = []
         # last part of URL filename
@@ -57,12 +53,12 @@ class FtpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         order: login, changing directory, list the file.
         """
         # proxy support (we support only http)
-        self.set_proxy(self.consumer.config["proxy"].get(self.scheme))
+        self.set_proxy(self.consumer.config("proxy").get(self.scheme))
         if self.proxy:
             # using a (HTTP) proxy
             http = httpurl.HttpUrl(self.base_url,
                   self.recursion_level,
-                  self.consumer.config,
+                  self.consumer,
                   parent_url=self.parent_url,
                   base_ref=self.base_ref,
                   line=self.line,
@@ -91,7 +87,7 @@ class FtpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         # ready to connect
         _user, _password = self.get_user_password()
         key = ("ftp", self.urlparts[1], _user, _password)
-        conn = self.consumer.cache.get_connection(key)
+        conn = self.consumer.get_connection(key)
         if conn is not None and conn.sock is not None:
             # reuse cached FTP connection
             self.url_connection = conn
@@ -146,7 +142,8 @@ class FtpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         if "%s/" % self.filename in files:
             if not self.url.endswith('/'):
                 self.add_warning(
-                         _("Missing trailing directory slash in ftp url."))
+                         _("Missing trailing directory slash in ftp url."),
+                         tag="ftp-missing-slash")
                 self.url += '/'
             return
         raise ftplib.error_perm, "550 File not found"
@@ -249,6 +246,6 @@ class FtpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         # add to cached connections
         _user, _password = self.get_user_password()
         key = ("ftp", self.urlparts[1], _user, _password)
-        cache_add = self.consumer.cache.add_connection
+        cache_add = self.consumer.add_connection
         cache_add(key, self.url_connection, DEFAULT_TIMEOUT_SECS)
         self.url_connection = None
