@@ -22,18 +22,18 @@ import urlparse
 import time
 import re
 import zlib
-import gzip
 import socket
 import cStringIO as StringIO
 import Cookie
 
 import linkcheck
 import linkcheck.url
+import linkcheck.gzip2
 import linkcheck.strformat
 import linkcheck.robotparser2
 import linkcheck.httplib2
 import httpheaders as headers
-import urlbase
+import internpaturl
 import proxysupport
 
 supportHttps = hasattr(linkcheck.httplib2, "HTTPSConnection") and \
@@ -44,24 +44,77 @@ _supported_encodings = ('gzip', 'x-gzip', 'deflate')
 # Amazon blocks all HEAD requests
 _is_amazon = re.compile(r'^www\.amazon\.(com|de|ca|fr|co\.(uk|jp))').search
 
+# Stolen from Python CVS urllib2.py
+# Mapping status codes to official W3C names
+httpresponses = {
+    100: 'Continue',
+    101: 'Switching Protocols',
+    
+    200: 'OK',
+    201: 'Created',
+    202: 'Accepted',
+    203: 'Non-Authoritative Information',
+    204: 'No Content',
+    205: 'Reset Content',
+    206: 'Partial Content',
+    
+    300: 'Multiple Choices',
+    301: 'Moved Permanently',
+    302: 'Found',
+    303: 'See Other',
+    304: 'Not Modified',
+    305: 'Use Proxy',
+    306: '(Unused)',
+    307: 'Temporary Redirect',
+    
+    400: 'Bad Request',
+    401: 'Unauthorized',
+    402: 'Payment Required',
+    403: 'Forbidden',
+    404: 'Not Found',
+    405: 'Method Not Allowed',
+    406: 'Not Acceptable',
+    407: 'Proxy Authentication Required',
+    408: 'Request Timeout',
+    409: 'Conflict',
+    410: 'Gone',
+    411: 'Length Required',
+    412: 'Precondition Failed',
+    413: 'Request Entity Too Large',
+    414: 'Request-URI Too Long',
+    415: 'Unsupported Media Type',
+    416: 'Requested Range Not Satisfiable',
+    417: 'Expectation Failed',
+    
+    500: 'Internal Server Error',
+    501: 'Not Implemented',
+    502: 'Bad Gateway',
+    503: 'Service Unavailable',
+    504: 'Gateway Timeout',
+    505: 'HTTP Version Not Supported',
+}
 
-class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
+class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
     """
     Url link with http scheme.
     """
 
-    def __init__ (self, base_url, recursion_level, consumer,
-                  parent_url=None, base_ref=None, line=0, column=0, name=u""):
+    def reset (self):
         """
-        Initialize basic url data and HTTP specific variables.
+        Initialize HTTP specific variables.
         """
-        super(HttpUrl, self).__init__(base_url, recursion_level, consumer,
-               parent_url=parent_url, base_ref=base_ref, line=line,
-               column=column, name=name)
+        super(HttpUrl, self).reset()
         self.max_redirects = 5
         self.has301status = False
-        self.no_anchor = False # remove anchor in request url
+        # some servers do not support anchors in requests
+        # this flag tells us to remove the anchor in request url
+        self.no_anchor = False
+        # flag if check had to fallback from HEAD to GET method
+        self.fallback_get = False
+        # flag if connection is persistent
         self.persistent = False
+        # URLs seen through 301/302 redirections
+        self.aliases = []
 
     def allows_robots (self, url):
         """
@@ -75,8 +128,8 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         """
         roboturl = self.get_robots_txt_url()
         user, password = self.get_user_password()
-        return self.consumer.cache.robots_txt_allows_url(roboturl, url,
-                                                         user, password)
+        return self.consumer.robots_txt_allows_url(roboturl, url,
+                                                   user, password)
 
     def check_connection (self):
         """
@@ -94,71 +147,58 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
             be fulfilled
           - 5xx: Server Error - The server failed to fulfill an apparently
             valid request
-
-        The individual values of the numeric status codes defined for
-        HTTP/1.0, and an example set of corresponding Reason-Phrase's, are
-        presented below. The reason phrases listed here are only recommended
-        -- they may be replaced by local equivalents without affecting the
-        protocol. These codes are fully defined in Section 9.
-        Status-Code    = "200"   ; OK
-        | "201"   ; Created
-        | "202"   ; Accepted
-        | "204"   ; No Content
-        | "301"   ; Moved Permanently
-        | "302"   ; Moved Temporarily
-        | "304"   ; Not Modified
-        | "305"   ; Use Proxy
-        | "400"   ; Bad Request
-        | "401"   ; Unauthorized
-        | "403"   ; Forbidden
-        | "404"   ; Not Found
-        | "405"   ; Method not allowed
-        | "407"   ; Proxy Authentication Required
-        | "500"   ; Internal Server Error
-        | "501"   ; Not Implemented
-        | "502"   ; Bad Gateway
-        | "503"   ; Service Unavailable
-        | extension-code
         """
         # set the proxy, so a 407 status after this is an error
-        self.set_proxy(self.consumer.config["proxy"].get(self.scheme))
+        self.set_proxy(self.consumer.config("proxy").get(self.scheme))
         # initialize check data
         self.headers = None
         self.auth = None
         self.cookies = []
         # check robots.txt
         if not self.allows_robots(self.url):
+            # remove all previously stored results
             self.add_warning(
-                       _("Access denied by robots.txt, checked only syntax."))
+                       _("Access denied by robots.txt, checked only syntax."),
+                       tag="http-robots-denied")
+            self.set_result(u"syntax OK")
             return
         # check for amazon server quirk
         if _is_amazon(self.urlparts[1]):
-            self.add_warning(_("Amazon servers block HTTP HEAD requests, "
-                               "using GET instead."))
+            self.add_info(_("Amazon servers block HTTP HEAD requests, "
+                            "using GET instead."))
             self.method = "GET"
         else:
             # first try with HEAD
             self.method = "HEAD"
         # check the http connection
-        response, fallback_GET = self.check_http_connection()
-        # check url warnings
-        effectiveurl = urlparse.urlunsplit(self.urlparts)
-        if self.url != effectiveurl:
-            self.add_warning(_("Effective URL %s.") % effectiveurl)
-            self.url = effectiveurl
+        response = self.check_http_connection()
+        if self.headers and self.headers.has_key("Server"):
+            server = self.headers['Server']
+        else:
+            server = _("unknown")
+        if self.fallback_get:
+            self.add_info(_("Server %r did not support HEAD request; "\
+                            "a GET request was used instead.") % server)
+        if self.no_anchor:
+            self.add_warning(_("Server %r had no anchor support, removed"\
+                               " anchor from request.") % server,
+                             tag="http-no-anchor-support")
+        # redirections might have changed the URL
+        newurl = urlparse.urlunsplit(self.urlparts)
+        if self.url != newurl:
+            self.url = newurl
         # check response
-        self.check_response(response, fallback_GET)
+        if response:
+            self.check_response(response)
 
     def check_http_connection (self):
         """
         Check HTTP connection and return get response and a flag
         if the check algorithm had to fall back to the GET method.
 
-        @return: (response, fallback_GET)
-        @rtype: tuple (HttpResponse, bool)
+        @return: response or None if url is already handled
+        @rtype: HttpResponse or None
         """
-        # flag if second try should be done with GET
-        fallback_GET = False
         while True:
             try:
                 response = self._get_http_response()
@@ -167,7 +207,7 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                 if self.method == "HEAD":
                     self.method = "GET"
                     self.aliases = []
-                    fallback_GET = True
+                    self.fallback_get = True
                     continue
                 raise
             if response.reason:
@@ -187,7 +227,7 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                     self.set_result(
                          _("Enforced proxy %r ignored, aborting.") % newproxy,
                          valid=False)
-                    return response, fallback_GET
+                    return response
                 response = self._get_http_response()
                 # restore old proxy settings
                 self.proxy, self.proxyauth = oldproxy
@@ -198,22 +238,22 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                 if self.method == "HEAD":
                     self.method = "GET"
                     self.aliases = []
-                    fallback_GET = True
+                    self.fallback_get = True
                     continue
                 raise
             if tries == -1:
                 linkcheck.log.debug(linkcheck.LOG_CHECK, "already handled")
-                return response, fallback_GET
+                return None
             if tries >= self.max_redirects:
                 if self.method == "HEAD":
                     # Microsoft servers tend to recurse HEAD requests
                     self.method = "GET"
                     self.aliases = []
-                    fallback_GET = True
+                    self.fallback_get = True
                     continue
                 self.set_result(_("more than %d redirections, aborting") % \
                                 self.max_redirects, valid=False)
-                return response, fallback_GET
+                return response
             # user authentication
             if response.status == 401:
                 if not self.auth:
@@ -228,11 +268,10 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                 if self.headers and self.urlparts[4] and not self.no_anchor:
                     self.no_anchor = True
                     continue
-                # retry with GET
+                # retry with GET (but do not set fallback flag)
                 if self.method == "HEAD":
                     self.method = "GET"
                     self.aliases = []
-                    fallback_GET = True
                     continue
             elif self.headers and self.method == "HEAD":
                 # test for HEAD support
@@ -244,10 +283,10 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                     # Zope server could not get Content-Type with HEAD
                     self.method = "GET"
                     self.aliases = []
-                    fallback_GET = True
+                    self.fallback_get = True
                     continue
             break
-        return response, fallback_GET
+        return response
 
     def follow_redirections (self, response):
         """
@@ -270,15 +309,18 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
             linkcheck.log.debug(linkcheck.LOG_CHECK, "Norm redirected to %r",
                                 redirected)
             # check extern filter again
-            self.extern = self._get_extern(redirected)
-            if self.is_extern():
+            self.set_extern(redirected)
+            if self.extern[0] and self.extern[0]:
                 self.add_info(
                           _("Outside of domain filter, checked only syntax."))
+                self.set_result(u"filtered")
                 return -1, response
             # check robots.txt allowance again
             if not self.allows_robots(redirected):
                 self.add_warning(
-                       _("Access denied by robots.txt, checked only syntax."))
+                       _("Access denied by robots.txt, checked only syntax."),
+                       tag="http-robots-denied")
+                self.set_result(u"syntax OK")
                 return -1, response
             # see about recursive redirect
             all_seen = [self.cache_url_key] + self.aliases
@@ -300,28 +342,25 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                 if not self.has301status:
                     self.add_warning(
                            _("HTTP 301 (moved permanent) encountered: you"
-                             " should update this link."))
-                    if not (self.url.endswith('/') or \
-                       self.url.endswith('.html')):
-                        self.add_warning(
-                       _("A HTTP 301 redirection occured and the URL has no "
-                     "trailing / at the end. All URLs which point to (home) "
-                     "directories should end with a / to avoid redirection."))
+                             " should update this link."),
+                           tag="http-moved-permanent")
                     self.has301status = True
             # check cache again on the changed URL
-            if self.consumer.cache.checked_redirect(redirected, self):
+            if self.consumer.checked_redirect(redirected, self):
                 return -1, response
             # check if we still have a http url, it could be another
             # scheme, eg https or news
             if self.urlparts[0] != "http":
                 self.add_warning(
                            _("HTTP redirection to non-http url encountered; "
-                             "the original url was %r.") % self.url)
+                             "the original url was %r.") % self.url,
+                           tag="http-wrong-redirect")
                 # make new Url object
                 newobj = linkcheck.checker.get_url_from(
                           redirected, self.recursion_level, self.consumer,
                           parent_url=self.parent_url, base_ref=self.base_ref,
-                          line=self.line, column=self.column, name=self.name)
+                          line=self.line, column=self.column, name=self.name,
+                          cmdline=False)
                 newobj.warning = self.warning
                 newobj.info = self.info
                 # append new object to queue
@@ -333,7 +372,7 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
             tries += 1
         return tries, response
 
-    def check_response (self, response, fallback_GET):
+    def check_response (self, response):
         """
         Check final result and log it.
         """
@@ -341,32 +380,24 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
             self.set_result(u"%r %s" % (response.status, response.reason),
                             valid=False)
         else:
-            if self.headers and self.headers.has_key("Server"):
-                server = self.headers['Server']
-            else:
-                server = _("unknown")
-            if fallback_GET:
-                self.add_warning(_("Server %r did not support HEAD request,"\
-                                   " used GET for checking.") % server)
-            if self.no_anchor:
-                self.add_warning(_("Server %r had no anchor support, removed"\
-                                   " anchor from request.") % server)
             if response.status == 204:
                 # no content
                 self.add_warning(
-                            linkcheck.strformat.unicode_safe(response.reason))
+                            linkcheck.strformat.unicode_safe(response.reason),
+                            tag="http-empty-content")
             # store cookies for valid links
-            if self.consumer.config['cookies']:
+            if self.consumer.config('cookies'):
                 for c in self.cookies:
                     self.add_info(_("Store cookie: %s.") % c)
                 try:
-                    out = self.consumer.cache.store_cookies(self.headers,
-                                                            self.urlparts[1])
+                    out = self.consumer.store_cookies(self.headers,
+                                                      self.urlparts[1])
                     for h in out:
                         self.add_info(linkcheck.strformat.unicode_safe(h))
                 except Cookie.CookieError, msg:
                     self.add_warning(_("Could not store cookies: %(msg)s.") %
-                                     {'msg': str(msg)})
+                                     {'msg': str(msg)},
+                                     tag="http-cookie-store-error")
             if response.status >= 200:
                 self.set_result(u"%r %s" % (response.status, response.reason))
             else:
@@ -417,9 +448,9 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                                       linkcheck.configuration.UserAgent)
         self.url_connection.putheader("Accept-Encoding",
                                   "gzip;q=1.0, deflate;q=0.9, identity;q=0.5")
-        if self.consumer.config['cookies']:
-            self.cookies = self.consumer.cache.get_cookies(self.urlparts[1],
-                                                           self.urlparts[2])
+        if self.consumer.config('cookies'):
+            self.cookies = self.consumer.get_cookies(self.urlparts[1],
+                                                     self.urlparts[2])
             for c in self.cookies:
                 self.url_connection.putheader("Cookie", c)
         self.url_connection.endheaders()
@@ -427,6 +458,9 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         self.persistent = headers.http_persistent(response)
         self.timeout = headers.http_timeout(response)
         self.headers = response.msg
+        # If possible, use official W3C HTTP response name
+        if response.status in httpresponses:
+            response.reason = httpresponses[response.status]
         return response
 
     def get_http_object (self, host, scheme):
@@ -442,7 +476,7 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         """
         _user, _password = self.get_user_password()
         key = (scheme, self.urlparts[1], _user, _password)
-        conn = self.consumer.cache.get_connection(key)
+        conn = self.consumer.get_connection(key)
         if conn is not None:
             linkcheck.log.debug(linkcheck.LOG_CHECK,
                                 "reuse cached HTTP(S) connection %s", conn)
@@ -481,11 +515,12 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                     if encoding == 'deflate':
                         f = StringIO.StringIO(zlib.decompress(self.data))
                     else:
-                        f = gzip.GzipFile('', 'rb', 9,
+                        f = linkcheck.gzip2.GzipFile('', 'rb', 9,
                                           StringIO.StringIO(self.data))
                 except zlib.error, msg:
                     self.add_warning(_("Decompress error %(err)s") % \
-                                     {"err": str(msg)})
+                                     {"err": str(msg)},
+                                     tag="http-decompress-error")
                     f = StringIO.StringIO(self.data)
                 self.data = f.read()
             self.downloadtime = time.time() - t
@@ -506,7 +541,8 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         encoding = headers.get_content_encoding(self.headers)
         if encoding and encoding not in _supported_encodings and \
            encoding != 'identity':
-            self.add_warning(_('Unsupported content encoding %r.') % encoding)
+            self.add_warning(_('Unsupported content encoding %r.') % encoding,
+                             tag="http-unsupported-encoding")
             return False
         return True
 
@@ -534,7 +570,8 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         encoding = headers.get_content_encoding(self.headers)
         if encoding and encoding not in _supported_encodings and \
            encoding != 'identity':
-            self.add_warning(_('Unsupported content encoding %r.') % encoding)
+            self.add_warning(_('Unsupported content encoding %r.') % encoding,
+                             tag="http-unsupported-encoding")
             return False
         return True
 
@@ -569,7 +606,7 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         # add to cached connections
         _user, _password = self.get_user_password()
         key = ("http", self.urlparts[1], _user, _password)
-        cache_add = self.consumer.cache.add_connection
+        cache_add = self.consumer.add_connection
         # note: only cache the connection when it is persistent
         # and all pending content has been received
         if not self.persistent or not self.has_content or \
