@@ -24,13 +24,14 @@ import urlparse
 import urllib2
 import urllib
 import time
-import traceback
+import errno
 import socket
 import select
-import codecs
+import traceback
 
 import linkcheck
 import linkcheck.linkparse
+import linkcheck.checker
 import linkcheck.strformat
 import linkcheck.containers
 import linkcheck.log
@@ -38,54 +39,11 @@ import linkcheck.httplib2
 import linkcheck.HtmlParser.htmlsax
 
 
-stderr = codecs.getwriter("iso8859-1")(sys.stderr, errors="ignore")
-
-def internal_error ():
-    """
-    Print internal error message to stderr.
-    """
-    print >> stderr, os.linesep
-    print >> stderr, _("""********** Oops, I did it again. *************
-
-You have found an internal error in LinkChecker. Please write a bug report
-at http://sourceforge.net/tracker/?func=add&group_id=1913&atid=101913
-or send mail to %s and include the following information:
-- the URL or file you are testing
-- your commandline arguments and/or configuration.
-- the output of a debug run with option "-Dall" of the executed command
-- the system information below.
-
-Disclosing some of the information above due to privacy reasons is ok.
-I will try to help you nonetheless, but you have to give me something
-I can work with ;) .
-""") % linkcheck.configuration.Email
-    etype, value = sys.exc_info()[:2]
-    print >> stderr, etype, value
-    traceback.print_exc()
-    print_app_info()
-    print >> stderr, os.linesep, \
-            _("******** LinkChecker internal error, bailing out ********")
-    sys.exit(1)
-
-
-def print_app_info ():
-    """
-    Print system and application info to stderr.
-    """
-    print >> stderr, _("System info:")
-    print >> stderr, linkcheck.configuration.App
-    print >> stderr, _("Python %s on %s") % (sys.version, sys.platform)
-    for key in ("LC_ALL", "LC_MESSAGES",  "http_proxy", "ftp_proxy"):
-        value = os.getenv(key)
-        if value is not None:
-            print >> stderr, key, "=", repr(value)
-
-
 def urljoin (parent, url, scheme):
     """
     If url is relative, join parent and url. Else leave url as-is.
 
-    @return join url
+    @return joined url
     """
     if url.startswith(scheme+":"):
         return url
@@ -112,6 +70,17 @@ class UrlBase (object):
         @param column: column number of url in parent content
         @param name: name of url or empty
         """
+        self.init(base_ref, base_url, parent_url, recursion_level,
+                  consumer, line, column, name)
+        self.reset()
+        self.check_syntax()
+
+
+    def init (self, base_ref, base_url, parent_url, recursion_level,
+              consumer, line, column, name):
+        """
+        Initialize internal data.
+        """
         self.base_ref = base_ref
         # note that self.base_url must not be modified
         self.base_url = base_url
@@ -131,6 +100,7 @@ class UrlBase (object):
         # assume file link if no scheme is found
         self.scheme = url.split(":", 1)[0] or "file"
 
+    def reset (self):
         # self.url is constructed by self.build_url() out of base_url
         # and (base_ref or parent) as absolute and normed url.
         # This the real url we use when checking so it also referred to
@@ -140,14 +110,15 @@ class UrlBase (object):
         self.urlparts = None
         # the anchor part of url
         self.anchor = None
-        # the result message string
+        # the result message string and flag
         self.result = u""
+        self.has_result = False
         # cached or not
         self.cached = False
         # valid or not
         self.valid = True
         # list of warnings (without duplicates)
-        self.warning = linkcheck.containers.SetList()
+        self.warnings = linkcheck.containers.SetList()
         # list of infos (without duplicates)
         self.info = linkcheck.containers.SetList()
         # download time
@@ -158,7 +129,6 @@ class UrlBase (object):
         self.checktime = 0
         # connection object
         self.url_connection = None
-        self.extern = (1, 0)
         # data of url content
         self.data = None
         # if data is filled
@@ -166,17 +136,26 @@ class UrlBase (object):
         # cache keys, are set by build_url() calling set_cache_keys()
         self.cache_url_key = None
         self.cache_content_key = None
-        # Store a list of aliases since the same resource can be known
-        # under several URLs. This is used for example on HTTP 30[12]
-        # redirections.
-        # Note that aliases are saved as-is, not through the cache-key-
-        # generating method set_cache_keys().
-        self.aliases = []
+        # extern flags (is_extern, is_strict), both enabled as default
+        self.extern = (1, 1)
+        # flag if the result should be cached
+        self.caching = True
 
     def set_result (self, msg, valid=True):
         """
         Set result string and validity.
         """
+        if self.has_result:
+            linkcheck.log.warn(linkcheck.LOG_CHECK,
+              "Double result %r (previous %r) for %s", msg, self.result, self)
+        else:
+            self.has_result = True
+        if not isinstance(msg, unicode):
+            linkcheck.log.warn(linkcheck.LOG_CHECK,
+                               "Non-unicode result for %s: %r", self, msg)
+        elif not msg:
+            linkcheck.log.warn(linkcheck.LOG_CHECK,
+                               "Empty result for %s", self)
         self.result = msg
         self.valid = valid
 
@@ -204,11 +183,11 @@ class UrlBase (object):
         """
         return False
 
-    def add_warning (self, s):
+    def add_warning (self, s, tag=None):
         """
         Add a warning string.
         """
-        self.warning.append(s)
+        self.warnings.append((tag, s))
 
     def add_info (self, s):
         """
@@ -221,7 +200,7 @@ class UrlBase (object):
         Fill attributes from cache data.
         """
         self.result = cache_data["result"]
-        self.warning.extend(cache_data["warning"])
+        self.warnings.extend(cache_data["warnings"])
         self.info.extend(cache_data["info"])
         self.valid = cache_data["valid"]
         self.dltime = cache_data["dltime"]
@@ -233,7 +212,7 @@ class UrlBase (object):
         Return all data values that should be put in the cache.
         """
         return {"result": self.result,
-                "warning": self.warning,
+                "warnings": self.warnings,
                 "info": self.info,
                 "valid": self.valid,
                 "dltime": self.dltime,
@@ -251,13 +230,11 @@ class UrlBase (object):
         linkcheck.log.debug(linkcheck.LOG_CACHE, "Content cache key %r",
                             self.cache_content_key)
         # construct cache key
-        if self.consumer.config["anchorcaching"] and \
-           self.consumer.config["anchors"]:
+        if self.consumer.config("anchorcaching") and \
+           self.consumer.config("anchors"):
             # do not ignore anchor
             parts = self.urlparts[:]
             parts[4] = self.anchor
-            if self.userinfo:
-                parts[1] = self.userinfo+u"@"+parts[1]
             self.cache_url_key = urlparse.urlunsplit(parts)
         else:
             # no anchor caching
@@ -272,20 +249,28 @@ class UrlBase (object):
         url syntax. Success enables further checking, failure
         immediately logs this url. Syntax checks must not
         use any network resources.
+
+        @return: True if syntax is correct, else False.
+        @rtype: bool
         """
         linkcheck.log.debug(linkcheck.LOG_CHECK, "checking syntax")
-        if not self.base_url:
+        if (self.base_url is None) or \
+           (not self.base_url and not self.parent_url):
             self.set_result(_("URL is empty"), valid=False)
-            return False
+            return
         try:
             self.build_url()
+            # check url warnings
+            effectiveurl = urlparse.urlunsplit(self.urlparts)
+            if self.url != effectiveurl:
+                self.add_warning(_("Effective URL %r.") % effectiveurl,
+                                 tag="url-effective-url")
+                self.url = effectiveurl
         except linkcheck.LinkCheckerError, msg:
             self.set_result(linkcheck.strformat.unicode_safe(msg),
                             valid=False)
-            return False
+            return
         self.set_cache_keys()
-        self.extern = self._get_extern(self.url)
-        return True
 
     def build_url (self):
         """
@@ -295,13 +280,14 @@ class UrlBase (object):
         # norm base url
         base_url, is_idn = linkcheck.url.url_norm(self.base_url)
         if is_idn:
-            self.add_warning(_("""URL %s has a unicode domain name which
+            self.add_warning(_("""URL %r has a unicode domain name which
                           is not yet widely supported. You should use
-                          the URL %s instead.""") % (self.base_url, base_url))
+                          the URL %r instead.""") % (self.base_url, base_url),
+                          tag="url-unicode-domain")
         elif self.base_url != base_url:
             self.add_warning(
-              _("Base URL is not properly normed. Normed url is %(url)s.") % \
-               {'url': base_url})
+              _("Base URL is not properly normed. Normed URL is %(url)s.") % \
+               {'url': base_url}, tag="url-unnormed")
         # make url absolute
         if self.base_ref:
             # use base reference as parent url
@@ -311,17 +297,29 @@ class UrlBase (object):
                                         self.scheme)
             self.url = urljoin(self.base_ref, base_url, self.scheme)
         elif self.parent_url:
-            self.url = urljoin(self.parent_url, base_url, self.scheme)
+            # strip the parent url query and anchor
+            urlparts = list(urlparse.urlsplit(self.parent_url))
+            urlparts[3] = urlparts[4] = ""
+            parent_url = urlparse.urlunsplit(urlparts)
+            self.url = urljoin(parent_url, base_url, self.scheme)
         else:
             self.url = base_url
+        # note: urljoin can unnorm the url path, so norm it again
+        urlparts = list(urlparse.urlsplit(self.url))
+        if urlparts[2]:
+            urlparts[2] = linkcheck.url.collapse_segments(urlparts[2])
+        self.url = urlparse.urlunsplit(urlparts)
         # split into (modifiable) list
         self.urlparts = linkcheck.strformat.url_unicode_split(self.url)
         # and unsplit again
         self.url = urlparse.urlunsplit(self.urlparts)
         # check userinfo@host:port syntax
         self.userinfo, host = urllib.splituser(self.urlparts[1])
-        # set host lowercase and without userinfo
-        self.urlparts[1] = host.lower()
+        # set host lowercase
+        if self.userinfo:
+            self.urlparts[1] = "%s@%s" % (self.userinfo, host.lower())
+        else:
+            self.urlparts[1] = host.lower()
         # safe anchor for later checking
         self.anchor = self.urlparts[4]
         self.host, self.port = urllib.splitport(host)
@@ -335,6 +333,8 @@ class UrlBase (object):
         """
         Main check function for checking this URL.
         """
+        if self.consumer.config("trace"):
+            linkcheck.log.trace()
         try:
             self.local_check()
             self.consumer.checked(self)
@@ -352,7 +352,7 @@ class UrlBase (object):
             raise
         except:
             self.consumer.interrupted(self)
-            internal_error()
+            linkcheck.checker.internal_error()
 
     def add_country_info (self):
         """
@@ -367,13 +367,14 @@ class UrlBase (object):
         Local check function can be overridden in subclasses.
         """
         linkcheck.log.debug(linkcheck.LOG_CHECK, "Checking %s", self)
-        if self.recursion_level and self.consumer.config['wait']:
+        if self.recursion_level and self.consumer.config('wait'):
             linkcheck.log.debug(linkcheck.LOG_CHECK,
                                 "sleeping for %d seconds",
-                                self.consumer.config['wait'])
-            time.sleep(self.consumer.config['wait'])
+                                self.consumer.config('wait'))
+            time.sleep(self.consumer.config('wait'))
         t = time.time()
-        if self.is_extern():
+        self.set_extern(self.url)
+        if self.extern[0] and self.extern[1]:
             self.add_info(_("Outside of domain filter, checked only syntax."))
             return
 
@@ -382,31 +383,27 @@ class UrlBase (object):
         try:
             self.check_connection()
             self.add_country_info()
-            if self.consumer.config["anchors"]:
+            if self.consumer.config("anchors"):
                 self.check_anchors()
         except tuple(linkcheck.checker.ExcList):
-            etype, evalue, etb = sys.exc_info()
-            linkcheck.log.debug(linkcheck.LOG_CHECK, "exception %s",
-                                traceback.format_tb(etb))
+            value = self.handle_exception()
             # make nicer error msg for unknown hosts
-            if isinstance(evalue, socket.error) and evalue[0] == -2:
-                evalue = _('Hostname not found')
+            if isinstance(value, socket.error) and value[0] == -2:
+                value = _('Hostname not found')
             # make nicer error msg for bad status line
-            if isinstance(evalue, linkcheck.httplib2.BadStatusLine):
-                evalue = _('Bad HTTP response %r') % str(evalue)
-            self.set_result(linkcheck.strformat.unicode_safe(evalue),
+            if isinstance(value, linkcheck.httplib2.BadStatusLine):
+                value = _('Bad HTTP response %r') % str(value)
+            self.set_result(linkcheck.strformat.unicode_safe(value),
                             valid=False)
 
         # check content
-        warningregex = self.consumer.config["warningregex"]
+        warningregex = self.consumer.config("warningregex")
         if warningregex and self.valid:
             linkcheck.log.debug(linkcheck.LOG_CHECK, "checking content")
             try:
                 self.check_content(warningregex)
             except tuple(linkcheck.checker.ExcList):
-                value, tb = sys.exc_info()[1:]
-                linkcheck.log.debug(linkcheck.LOG_CHECK, "exception %s",
-                                    traceback.format_tb(tb))
+                value = self.handle_exception()
                 self.set_result(linkcheck.strformat.unicode_safe(value),
                                 valid=False)
 
@@ -421,11 +418,9 @@ class UrlBase (object):
             # check content size
             self.check_size()
         except tuple(linkcheck.checker.ExcList):
-            value, tb = sys.exc_info()[1:]
-            linkcheck.log.debug(linkcheck.LOG_CHECK, "exception %s",
-                                traceback.format_tb(tb))
-            self.set_result(_("could not parse content: %r") % str(value),
-                            valid=False)
+            value = self.handle_exception()
+            self.add_warning(_("could not get content: %r") % str(value),
+                            tag="url-error-getting-content")
         # close
         self.close_connection()
 
@@ -443,6 +438,21 @@ class UrlBase (object):
             pass
         self.url_connection = None
 
+    def handle_exception (self):
+        """
+        An exception occurred. Log it and set the cache flag.
+        """
+        etype, value, tb = sys.exc_info()
+        linkcheck.log.debug(linkcheck.LOG_CHECK, "exception %s",
+                            traceback.format_tb(tb))
+        # note: etype must be the exact class, not a subclass
+        if (etype in linkcheck.checker.ExcNoCacheList) or \
+           (etype == socket.error and value[0]==errno.EBADF) or \
+            not value:
+            # EBADF occurs when operating on an already socket
+            self.caching = False
+        return value
+
     def check_connection (self):
         """
         The basic connection check uses urllib2.urlopen to initialize
@@ -464,8 +474,8 @@ class UrlBase (object):
         return self.valid and \
             self.is_parseable() and \
             self.can_get_content() and \
-            (self.consumer.config["recursionlevel"] < 0 or
-            self.recursion_level < self.consumer.config["recursionlevel"]) and \
+            (self.consumer.config("recursionlevel") < 0 or
+            self.recursion_level < self.consumer.config("recursionlevel")) and \
             not self.extern[0] and self.content_allows_robots()
 
     def content_allows_robots (self):
@@ -508,64 +518,34 @@ class UrlBase (object):
         for cur_anchor, line, column, name, base in h.urls:
             if cur_anchor == self.anchor:
                 return
-        self.add_warning(_("Anchor #%s not found.") % self.anchor)
+        self.add_warning(_("Anchor #%s not found.") % self.anchor,
+                         tag="url-anchor-not-found")
 
-    def is_extern (self):
+    def set_extern (self, url):
         """
-        Determine if this URL is extern or not.
+        Match URL against extern and intern link patterns. If no pattern
+        matches the URL is extern. Sets self.extern to a tuple (bool,
+        bool) with content (is_extern, is_strict).
 
-        @return: True if URL is extern, else False
-        @rtype: bool
+        @return: None
         """
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "extern=%s", self.extern)
-        return self.extern[0] and \
-           (self.consumer.config["externstrictall"] or self.extern[1])
-
-    def _get_extern (self, url):
-        """
-        Match URL against intern and extern link patterns, according
-        to the configured denyallow order.
-
-        @return: a tuple (is_extern, is_strict)
-        @rtype: tuple (bool, bool)
-        """
-        if not (self.consumer.config["externlinks"] or \
-           self.consumer.config["internlinks"]):
-            return (0, 0)
-        # deny and allow external checking
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "Url %r", url)
-        if self.consumer.config["denyallow"]:
-            for entry in self.consumer.config["externlinks"]:
-                linkcheck.log.debug(linkcheck.LOG_CHECK, "Extern entry %r",
-                                    entry)
-                match = entry['pattern'].search(url)
-                if (entry['negate'] and not match) or \
-                   (match and not entry['negate']):
-                    return (1, entry['strict'])
-            for entry in self.consumer.config["internlinks"]:
-                linkcheck.log.debug(linkcheck.LOG_CHECK, "Intern entry %r",
-                                    entry)
-                match = entry['pattern'].search(url)
-                if (entry['negate'] and not match) or \
-                   (match and not entry['negate']):
-                    return (0, 0)
-            return (0, 0)
-        else:
-            for entry in self.consumer.config["internlinks"]:
-                linkcheck.log.debug(linkcheck.LOG_CHECK, "Intern entry %r",
-                                    entry)
-                match = entry['pattern'].search(url)
-                if (entry['negate'] and not match) or \
-                   (match and not entry['negate']):
-                    return (0, 0)
-            for entry in self.consumer.config["externlinks"]:
-                linkcheck.log.debug(linkcheck.LOG_CHECK, "Extern entry %r",
-                                    entry)
-                match = entry['pattern'].search(url)
-                if (entry['negate'] and not match) or \
-                   (match and not entry['negate']):
-                    return (1, entry['strict'])
-            return (1, 0)
+        for entry in self.consumer.config("externlinks"):
+            match = entry['pattern'].search(url)
+            if (entry['negate'] and not match) or \
+               (match and not entry['negate']):
+                linkcheck.log.debug(linkcheck.LOG_CHECK, "Extern URL %r", url)
+                self.extern = (1, entry['strict'])
+                return
+        for entry in self.consumer.config("internlinks"):
+            match = entry['pattern'].search(url)
+            if (entry['negate'] and not match) or \
+               (match and not entry['negate']):
+                linkcheck.log.debug(linkcheck.LOG_CHECK, "Intern URL %r", url)
+                self.extern = (0, 0)
+                return
+        linkcheck.log.debug(linkcheck.LOG_CHECK, "Extern URL %r", url)
+        self.extern = (1, 0)
+        return
 
     def can_get_content (self):
         """
@@ -594,18 +574,20 @@ class UrlBase (object):
             return
         match = warningregex.search(self.get_content())
         if match:
-            self.add_warning(_("Found %r in link contents.") % match.group())
+            self.add_warning(_("Found %r in link contents.") % match.group(),
+                             tag="url-warnregex-found")
 
     def check_size (self):
         """
         If a maximum size was given, call this function to check it
         against the content size of this url.
         """
-        maxbytes = self.consumer.config["warnsizebytes"]
+        maxbytes = self.consumer.config("warnsizebytes")
         if maxbytes is not None and self.dlsize >= maxbytes:
             self.add_warning(_("Content size %s is larger than %s.") % \
                          (linkcheck.strformat.strsize(self.dlsize),
-                          linkcheck.strformat.strsize(maxbytes)))
+                          linkcheck.strformat.strsize(maxbytes)),
+                          tag="url-content-too-large")
 
     def parse_url (self):
         """
@@ -621,7 +603,7 @@ class UrlBase (object):
         Get tuple (user, password) from configured authentication.
         Both user and password can be None if not specified.
         """
-        for auth in self.consumer.config["authentication"]:
+        for auth in self.consumer.config("authentication"):
             if auth['pattern'].match(self.url):
                 return auth['user'], auth['password']
         return None, None
@@ -648,7 +630,8 @@ class UrlBase (object):
                 base_ref = h.base_ref
             url_data = linkcheck.checker.get_url_from(url,
                   self.recursion_level+1, self.consumer, parent_url=self.url,
-                  base_ref=base_ref, line=line, column=column, name=name)
+                  base_ref=base_ref, line=line, column=column, name=name,
+                  cmdline=False)
             self.consumer.append_url(url_data)
 
     def parse_opera (self):
@@ -668,7 +651,8 @@ class UrlBase (object):
                 if url:
                     url_data = linkcheck.checker.get_url_from(url,
                               self.recursion_level+1, self.consumer,
-                              parent_url=self.url, line=lineno, name=name)
+                              parent_url=self.url, line=lineno, name=name,
+                              cmdline=False)
                     self.consumer.append_url(url_data)
                 name = ""
 
@@ -685,7 +669,8 @@ class UrlBase (object):
                 continue
             url_data = linkcheck.checker.get_url_from(line,
                               self.recursion_level+1, self.consumer,
-                              parent_url=self.url, line=lineno)
+                              parent_url=self.url, line=lineno,
+                              cmdline=False)
             self.consumer.append_url(url_data)
 
     def parse_css (self):
@@ -700,7 +685,8 @@ class UrlBase (object):
                 url = linkcheck.strformat.unquote(mo.group("url").strip())
                 url_data = linkcheck.checker.get_url_from(url,
                              self.recursion_level+1, self.consumer,
-                             parent_url=self.url, line=lineno, column=column)
+                             parent_url=self.url, line=lineno, column=column,
+                             cmdline=False)
                 self.consumer.append_url(url_data)
 
     def serialized (self):
@@ -708,7 +694,8 @@ class UrlBase (object):
         Return serialized url check data as unicode string.
         """
         sep = linkcheck.strformat.unicode_safe(os.linesep)
-        assert isinstance(self.base_url, unicode), self
+        if self.base_url is not None:
+            assert isinstance(self.base_url, unicode), self
         if self.parent_url is not None:
             assert isinstance(self.parent_url, unicode), self
         if self.base_ref is not None:
@@ -726,6 +713,15 @@ class UrlBase (object):
             u"name=%r" % self.name,
            ])
 
+    def get_intern_pattern (self):
+        """
+        Get pattern for intern URL matching.
+
+        @return non-empty regex pattern or None
+        @rtype String or None
+        """
+        return None
+
     def __str__ (self):
         """
         Get URL info.
@@ -734,7 +730,7 @@ class UrlBase (object):
         @rtype: string
         """
         s = self.serialized()
-        return self.consumer.config['logger'].encode(s)
+        return self.consumer.config('logger').encode(s)
 
     def __repr__ (self):
         """
