@@ -41,7 +41,7 @@ AppName = u"LinkChecker"
 App = AppName+u" "+Version
 Author =  _linkchecker_configdata.author
 HtmlAuthor = Author.replace(u' ', u'&nbsp;')
-Copyright = u"Copyright  2000-2005 "+Author
+Copyright = u"Copyright (C) 2000-2005 "+Author
 HtmlCopyright = u"Copyright &copy; 2000-2005 "+HtmlAuthor
 AppInfo = App+u"              "+Copyright
 HtmlAppInfo = App+u", "+HtmlCopyright
@@ -73,16 +73,17 @@ class Configuration (dict):
         Initialize the default options.
         """
         super(Configuration, self).__init__()
+        self['debug'] = False
+        self['trace'] = False
         self["verbose"] = False
-        self["warnings"] = False
+        self["warnings"] = True
+        self["ignorewarnings"] = []
         self['quiet'] = False
         self["anchors"] = False
         self["anchorcaching"] = True
-        self["externstrictall"] = False
         self["externlinks"] = []
         self["internlinks"] = []
         self["noproxyfor"] = []
-        self["denyallow"] = False
         self["interactive"] = False
         # on ftp, password is set by Pythons ftplib
         self["authentication"] = []
@@ -137,6 +138,9 @@ class Configuration (dict):
         self['xml'] = {
             "filename": "linkchecker-out.xml",
         }
+        self['gxml'] = {
+            "filename": "linkchecker-out.gxml",
+        }
         self['dot'] = {
             "filename": "linkchecker-out.dot",
             "encoding": "ascii",
@@ -175,7 +179,7 @@ class Configuration (dict):
         handler = linkcheck.ansicolor.ColoredStreamHandler(strm=sys.stderr)
         handler.setFormatter(logging.Formatter("%(levelname)s %(message)s"))
         logging.getLogger(linkcheck.LOG).addHandler(handler)
-        if debug is not None:
+        if debug:
             self['debug'] = True
             # disable threading if no thread debugging
             if "thread" not in debug:
@@ -229,9 +233,13 @@ class Configuration (dict):
         if not cfiles:
             # system wide config settings
             config_dir = _linkchecker_configdata.config_dir
-            cfiles.append(normpath(os.path.join(config_dir, "linkcheckerrc")))
+            path = normpath(os.path.join(config_dir, "linkcheckerrc"))
+            if os.path.exists(path):
+                cfiles.append(path)
             # per user config settings
-            cfiles.append(normpath("~/.linkchecker/linkcheckerrc"))
+            path = normpath("~/.linkchecker/linkcheckerrc")
+            if os.path.exists(path):
+                cfiles.append(path)
         self.read_config(cfiles)
         # re-init logger
         self['logger'] = self.logger_new('text')
@@ -266,16 +274,12 @@ class Configuration (dict):
                     except ConfigParser.Error, msg:
                         linkcheck.log.debug(linkcheck.LOG_CHECK, msg)
                 try:
-                    self[key]['fields'] = [f.strip() \
-                            for f in cfgparser.get(key, 'fields').split(',')]
+                    self[key]['parts'] = [f.strip() for f in \
+                         cfgparser.get(key, 'parts').split(',')]
                 except ConfigParser.Error, msg:
                     linkcheck.log.debug(linkcheck.LOG_CHECK, msg)
         try:
-            logger = cfgparser.get(section, "log")
-            if linkcheck.Loggers.has_key(logger):
-                self['logger'] = self.logger_new(logger)
-            else:
-                linkcheck.log.warn(_("invalid log option %r"), logger)
+            self["warnings"] = cfgparser.getboolean(section, "warnings")
         except ConfigParser.Error, msg:
             linkcheck.log.debug(linkcheck.LOG_CHECK, msg)
         try:
@@ -295,7 +299,11 @@ class Configuration (dict):
         except ConfigParser.Error, msg:
             linkcheck.log.debug(linkcheck.LOG_CHECK, msg)
         try:
-            self["warnings"] = cfgparser.getboolean(section, "warnings")
+            logger = cfgparser.get(section, "log")
+            if linkcheck.Loggers.has_key(logger):
+                self['logger'] = self.logger_new(logger)
+            else:
+                linkcheck.log.warn(_("invalid log option %r"), logger)
         except ConfigParser.Error, msg:
             linkcheck.log.debug(linkcheck.LOG_CHECK, msg)
         try:
@@ -337,11 +345,6 @@ class Configuration (dict):
             self["recursionlevel"] = num
         except ConfigParser.Error, msg:
             linkcheck.log.debug(linkcheck.LOG_CHECK, msg)
-        try:
-            self["externstrictall"] = \
-                          cfgparser.getboolean(section, "externstrictall")
-        except ConfigParser.Error, msg:
-            linkcheck.log.debug(linkcheck.LOG_CHECK, msg)
         try:
             arg = cfgparser.get(section, "warningregex")
             if arg:
@@ -411,13 +414,31 @@ class Configuration (dict):
         try:
             i = 1
             while 1:
-                ctuple = cfgparser.get(section, "extern%d" % i).split()
+                ctuple = cfgparser.get(section, "nofollow%d" % i).split()
                 if len(ctuple)!=2:
                     linkcheck.log.error(
-                            _("extern%d: syntax error %s\n") % (i, ctuple))
+                            _("nofollow%d: syntax error %s\n") % (i, ctuple))
                     break
                 self["externlinks"].append(
-                    linkcheck.get_link_pat(ctuple[0], strict=int(ctuple[1])))
+                    linkcheck.get_link_pat(ctuple[0], strict=0))
+                i += 1
+        except ConfigParser.Error, msg:
+            linkcheck.log.debug(linkcheck.LOG_CHECK, msg)
+        try:
+            self['ignorewarnings'] = [f.strip() for f in \
+                 cfgparser.get(section, 'ignorewarnings').split(',')]
+        except ConfigParser.Error, msg:
+            linkcheck.log.debug(linkcheck.LOG_CHECK, msg)
+        try:
+            i = 1
+            while 1:
+                ctuple = cfgparser.get(section, "ignore%d" % i).split()
+                if len(ctuple)!=2:
+                    linkcheck.log.error(
+                            _("ignore%d: syntax error %s\n") % (i, ctuple))
+                    break
+                self["externlinks"].append(
+                    linkcheck.get_link_pat(ctuple[0], strict=1))
                 i += 1
         except ConfigParser.Error, msg:
             linkcheck.log.debug(linkcheck.LOG_CHECK, msg)
@@ -426,10 +447,6 @@ class Configuration (dict):
                linkcheck.get_link_pat(cfgparser.get(section, "internlinks")))
         except ConfigParser.Error, msg:
             linkcheck.log.debug(linkcheck.LOG_CHECK, msg)
-        try:
-            self["denyallow"] = cfgparser.getboolean(section, "denyallow")
-        except ConfigParser.Error, msg:
-            linkcheck.log.debug(linkcheck.LOG_CHECK, msg)
 
     def write_boolean_config (self, fp, boolopts):
         """
