@@ -22,10 +22,11 @@
 import socket
 import sets
 import sys
-import os
 import time
 import encodings.idna
 
+import linkcheck
+import linkcheck.log
 import linkcheck.dns.exception
 import linkcheck.dns.message
 import linkcheck.dns.name
@@ -492,6 +493,16 @@ class Resolver(object):
         finally:
             lm.Close()
 
+    def _compute_timeout(self, start):
+        now = time.time()
+        if now < start:
+            # Time going backwards is bad.  Just give up.
+            raise Timeout
+        duration = now - start
+        if duration >= self.lifetime:
+            raise Timeout
+        return min(self.lifetime - duration, self.timeout)
+
     def query(self, qname, rdtype=linkcheck.dns.rdatatype.A,
               rdclass=linkcheck.dns.rdataclass.IN, tcp=False):
         """Query nameservers to find the answer to the question.
@@ -553,18 +564,12 @@ class Resolver(object):
             # make a copy of the servers list so we can alter it later.
             #
             nameservers = self.nameservers[:]
+            backoff = 0.10
             while response is None:
                 if len(nameservers) == 0:
                     raise NoNameservers, "No DNS servers could answer the query"
                 for nameserver in nameservers:
-                    now = time.time()
-                    if now < start:
-                        # Time going backwards is bad.  Just give up.
-                        raise Timeout, "DNS query timed out"
-                    duration = now - start
-                    if duration >= self.lifetime:
-                        raise Timeout, "DNS query timed out"
-                    timeout = min(self.lifetime - duration, self.timeout)
+                    timeout = self._compute_timeout(start)
                     try:
                         if tcp:
                             response = linkcheck.dns.query.tcp(request, nameserver,
@@ -598,7 +603,26 @@ class Resolver(object):
                     if rcode == linkcheck.dns.rcode.NOERROR or \
                            rcode == linkcheck.dns.rcode.NXDOMAIN:
                         break
+                    #
+                    # We got a response, but we're not happy with the
+                    # rcode in it.  Remove the server from the mix if
+                    # the rcode isn't SERVFAIL.
+                    #
+                    if rcode != dns.rcode.SERVFAIL:
+                        nameservers.remove(nameserver)
                     response = None
+                #
+                # All nameservers failed!
+                #
+                if len(nameservers) > 0:
+                    #
+                    # But we still have servers to try.  Sleep a bit
+                    # so we don't pound them!
+                    #
+                    timeout = self._compute_timeout(start)
+                    sleep_time = min(timeout, backoff)
+                    backoff *= 2
+                    time.sleep(sleep_time)
             if response.rcode() == linkcheck.dns.rcode.NXDOMAIN:
                 continue
             all_nxdomain = False
@@ -656,4 +680,6 @@ def query(qname, rdtype=linkcheck.dns.rdatatype.A, rdclass=linkcheck.dns.rdatacl
     global default_resolver
     if default_resolver is None:
         default_resolver = Resolver()
+    linkcheck.log.debug(linkcheck.LOG_DNS,
+                        "Query %s %s %s", qname, rdtype, rdclass)
     return default_resolver.query(qname, rdtype, rdclass, tcp)
