@@ -39,9 +39,8 @@ def test_suite ():
     """
     Build and return a TestSuite.
     """
-    suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(TestAnchor))
-    return suite
+    return unittest.makeSuite(TestAnchor)
+
 
 if __name__ == '__main__':
     unittest.main()
