@@ -40,7 +40,9 @@ class TestMail (linkcheck.ftests.StandardTest):
           u"cache key mailto:calvin@users.sf.net,"
            u"calvin@users.sourceforge.net",
           u"real url %s" % url,
-          u"info Verified address: <calvin> is deliverable.",
+          u"info Verified address: <calvin@users.sf.net> is deliverable.",
+          u"info Verified address: <calvin@users.sourceforge.net> is "\
+           "deliverable.",
           u"valid",
         ]
         self.direct(url, resultlines)
@@ -51,7 +53,9 @@ class TestMail (linkcheck.ftests.StandardTest):
           u"cache key mailto:calvin@users.sf.net,"
            u"calvin@users.sourceforge.net",
           u"real url %s" % url,
-          u"info Verified address: <calvin> is deliverable.",
+          u"info Verified address: <calvin@users.sf.net> is deliverable.",
+          u"info Verified address: <calvin@users.sourceforge.net> is "\
+           "deliverable.",
           u"valid",
         ]
         self.direct(url, resultlines)
@@ -60,7 +64,7 @@ class TestMail (linkcheck.ftests.StandardTest):
             u"url %s" % url,
             u"cache key mailto:calvin@users.sf.net",
             u"real url %s" % url,
-            u"info Verified address: <calvin> is deliverable.",
+            u"info Verified address: <calvin@users.sf.net> is deliverable.",
             u"valid",
         ]
         self.direct(url, resultlines)
@@ -69,7 +73,7 @@ class TestMail (linkcheck.ftests.StandardTest):
             u"url %s" % url,
             u"cache key mailto:o'hara@users.sf.net",
             u"real url %s" % url,
-            u"info Verified address: <o'hara> is deliverable.",
+            u"warning Unverified address: <o'hara@users.sf.net> Unrouteable address.",
             u"valid",
         ]
         self.direct(url, resultlines)
@@ -80,9 +84,9 @@ class TestMail (linkcheck.ftests.StandardTest):
             u"cache key mailto:calvin@users.sf.net,"
              u"calvin_CC@users.sf.net,calvin_cc@users.sf.net",
             u"real url %s" % url,
-            u"info Verified address: <calvin> is deliverable.",
-            u"info Verified address: <calvin_cc> is deliverable.",
-            u"info Verified address: <calvin_CC> is deliverable.",
+           u"info Verified address: <calvin@users.sf.net> is deliverable.",
+            u"warning Unverified address: <calvin_cc@users.sf.net> Unrouteable address.",
+            u"warning Unverified address: <calvin_CC@users.sf.net> Unrouteable address.",
             u"valid",
         ]
         self.direct(url, resultlines)
@@ -103,7 +107,7 @@ class TestMail (linkcheck.ftests.StandardTest):
             u"cache key mailto:foo@foo-bar.de",
             u"real url %s" % url,
             u"info Unverified address: Cannot VRFY user; "
-             u"try RCPT to attempt delivery. But mail will be sent anyway.",
+            u"try RCPT to attempt delivery. But mail will be sent anyway.",
             u"valid",
         ]
         self.direct(url, resultlines)
@@ -119,9 +123,9 @@ class TestMail (linkcheck.ftests.StandardTest):
             u"url %s" % url,
             u"cache key mailto:calvin@users.sf.net",
             u"real url %s" % qurl,
-            u"info Verified address: <calvin> is deliverable.",
+            u"info Verified address: <calvin@users.sf.net> is deliverable.",
             u"warning Base URL is not properly normed. "
-             u"Normed url is %s." % qurl,
+             u"Normed URL is %s." % qurl,
             u"valid",
         ]
         self.direct(url, resultlines)
@@ -131,9 +135,9 @@ class TestMail (linkcheck.ftests.StandardTest):
             u"url %s" % url,
             u"cache key mailto:calvin@users.sf.net",
             u"real url %s" % qurl,
-            u"info Verified address: <calvin> is deliverable.",
+            u"info Verified address: <calvin@users.sf.net> is deliverable.",
             u"warning Base URL is not properly normed. "
-             u"Normed url is %s." % qurl,
+             u"Normed URL is %s." % qurl,
             u"valid",
         ]
         self.direct(url, resultlines)
@@ -167,9 +171,7 @@ def test_suite ():
     """
     Build and return a TestSuite.
     """
-    suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(TestMail))
-    return suite
+    return unittest.makeSuite(TestMail)
 
 
 if __name__ == '__main__':
