@@ -48,7 +48,7 @@ LinkTags = {
     'isindex':  [u'action'],
     'layer':    [u'background', u'src'],
     'link':     [u'href'],
-    'meta':     [u'content'],
+    'meta':     [u'content', u'href'],
     'object':   [u'classid', u'data', u'archive', u'usemap'],
     'q':        [u'cite'],
     'script':   [u'src', u'for'],
@@ -108,6 +108,19 @@ class TagFinder (object):
         """
         self._errorfun(msg, "fatal error")
 
+    def start_element (self, tag, attrs):
+        """
+        Does nothing, override in a subclass.
+        """
+        pass
+
+    def start_end_element (self, tag, attrs):
+        """
+        Delegate a combined start/end element (eg. <br/>) to
+        the start_element method. Ignore the end element part.
+        """
+        self.start_element(tag, attrs)
+
 
 class MetaRobotsFinder (TagFinder):
     """
@@ -123,7 +136,6 @@ class MetaRobotsFinder (TagFinder):
         self.index = True
         linkcheck.log.debug(linkcheck.LOG_CHECK, "meta robots finder")
 
-
     def start_element (self, tag, attrs):
         """
         Search for meta robots.txt "nofollow" and "noindex" flags.
@@ -135,6 +147,18 @@ class MetaRobotsFinder (TagFinder):
                 self.index = u'noindex' not in val
 
 
+def is_meta_url (attr, attrs):
+    res = False
+    if attr == "content":
+        equiv = attrs.get_true('http-equiv', u'').lower()
+	scheme = attrs.get_true('scheme', u'').lower()
+        res = equiv in (u'refresh',) or scheme in (u'dcterms.uri',)
+    if attr == "href":
+        rel = attrs.get_true('rel', u'').lower()
+        res = rel in (u'shortcut icon', u'icon')
+    return res
+
+
 class LinkFinder (TagFinder):
     """
     Find a list of links. After parsing, self.urls
@@ -174,10 +198,8 @@ class LinkFinder (TagFinder):
         for attr in tagattrs:
             if attr not in attrs:
                 continue
-            if tag == "meta":
-                refresh = attrs.get_true('http-equiv', u'').lower()
-                if refresh != 'refresh':
-                    continue
+            if tag == "meta" and not is_meta_url(attr, attrs):
+                continue
             # name of this link
             name = self.get_link_name(tag, attrs, attr)
             # possible codebase
@@ -185,7 +207,8 @@ class LinkFinder (TagFinder):
                 codebase = unquote(attrs.get_true('codebase', u''))
             else:
                 codebase = u''
-            value = unquote(attrs.get_true(attr, u''))
+            # note: value can be None
+            value = unquote(attrs.get(attr))
             # add link to url list
             self.add_link(tag, attr, value, name, codebase)
         linkcheck.log.debug(linkcheck.LOG_CHECK,
@@ -220,13 +243,15 @@ class LinkFinder (TagFinder):
         assert isinstance(attr, unicode), repr(attr)
         assert isinstance(name, unicode), repr(name)
         assert isinstance(base, unicode), repr(base)
-        assert isinstance(url, unicode), repr(url)
+        assert isinstance(url, unicode) or url is None, repr(url)
         urls = []
         # look for meta refresh
         if tag == u'meta':
             mo = refresh_re.match(url)
             if mo:
                 urls.append(mo.group("url"))
+            else:
+                urls.append(url)
         elif attr == u'style':
             for mo in css_url_re.finditer(url):
                 u = mo.group("url")
@@ -237,7 +262,7 @@ class LinkFinder (TagFinder):
             # no url found
             return
         for u in urls:
-            assert isinstance(u, unicode), repr(u)
+            assert isinstance(u, unicode) or u is None, repr(u)
             linkcheck.log.debug(linkcheck.LOG_CHECK,
               u"LinkParser add link %s %s %s %s %s", tag, attr, u, name, base)
             self.urls.append((u, self.parser.last_lineno(),
