@@ -18,6 +18,7 @@
 Output logging support for different formats.
 """
 
+import locale
 import sys
 import os
 import os.path
@@ -50,33 +51,37 @@ class Logger (object):
 
     def __init__ (self, **args):
         """
-        Initialize a logger, looking for field restrictions in kwargs.
+        Initialize a logger, looking for part restrictions in kwargs.
         """
-        # what log fields should be in output
-        self.logfields = None # log all fields
-        if args.has_key('fields'):
-            if "all" not in args['fields']:
-                # only log given fields
-                self.logfields = args['fields']
-        # number of spaces before log fields for alignment
+        # what log parts should be in output
+        self.logparts = None # log all parts
+        if args.has_key('parts'):
+            if "all" not in args['parts']:
+                # only log given parts
+                self.logparts = args['parts']
+        # number of spaces before log parts for alignment
         self.logspaces = {}
         # maximum indent of spaces for alignment
         self.max_indent = 0
+        # number of logged urls
+        self.number = 0
         # number of encountered errors
         self.errors = 0
+        # number of encountered warningss
+        self.warnings = 0
         # encoding of output
-        self.output_encoding = args.get("encoding", "iso-8859-1")
+        self.output_encoding = args.get("encoding", locale.getpreferredencoding())
 
     def init_fileoutput (self, args):
         """
         Initialize self.fd file descriptor from args.
         """
         if args.get('fileoutput'):
-            fname = args['filename']
-            path = os.path.dirname(fname)
+            filename = args['filename']
+            path = os.path.dirname(filename)
             if path and not os.path.isdir(path):
                 os.makedirs(path)
-            self.fd = file(args['filename'], "w")
+            self.fd = file(filename, "wb")
             self.close_fd = True
         elif args.has_key('fd'):
             self.fd = args['fd']
@@ -85,6 +90,15 @@ class Logger (object):
             self.fd = sys.stdout
             self.close_fd = False
 
+    def close_fileoutput (self):
+        """
+        Flush and close the file output denoted by self.fd.
+        """
+        self.flush()
+        if self.close_fd:
+            self.fd.close()
+        self.fd = None
+
     def encode (self, s):
         """
         Encode string with configured output encoding. Wrong encoded
@@ -110,7 +124,7 @@ class Logger (object):
 
     def comment (self, s, **args):
         """
-        Print a comment and a newline. This method just prints
+        Write a comment and a newline. This method just prints
         the given string.
         """
         self.writeln(s=s, **args)
@@ -140,24 +154,24 @@ class Logger (object):
         self.write(s)
         self.write(unicode(os.linesep), **args)
 
-    def has_field (self, name):
+    def has_part (self, name):
         """
-        See if given field name will be logged.
+        See if given part name will be logged.
         """
-        if self.logfields is None:
-            # log all fields
+        if self.logparts is None:
+            # log all parts
             return True
-        return name in self.logfields
+        return name in self.logparts
 
-    def field (self, name):
+    def part (self, name):
         """
-        Return translated field name.
+        Return translated part name.
         """
         return _(Fields[name])
 
     def spaces (self, name):
         """
-        Return indent of spaces for given field name.
+        Return indent of spaces for given part name.
         """
         return self.logspaces[name]
 
@@ -165,25 +179,37 @@ class Logger (object):
         """
         Start log output.
         """
-        # map with spaces between field name and value
-        if self.logfields is None:
-            fields = Fields.keys()
+        # map with spaces between part name and value
+        if self.logparts is None:
+            parts = Fields.keys()
         else:
-            fields = self.logfields
-        values = [self.field(x) for x in fields]
-        # maximum indent for localized log field names
+            parts = self.logparts
+        values = [self.part(x) for x in parts]
+        # maximum indent for localized log part names
         self.max_indent = max([len(x) for x in values])+1
-        for key in fields:
-            numspaces = (self.max_indent - len(self.field(key)))
+        for key in parts:
+            numspaces = (self.max_indent - len(self.part(key)))
             self.logspaces[key] = u" " * numspaces
 
-    def new_url (self, url_data):
+    def log_filter_url (self, url_data, do_print):
+        """
+        Log a new url with this logger if do_filter is True. Else
+        only update accounting data
+        """
+        self.number += 1
+        if not url_data.valid:
+            self.errors += 1
+        self.warnings += len(url_data.warnings)
+        if do_print:
+            self.log_url(url_data)
+
+    def log_url (self, url_data):
         """
         Log a new url with this logger.
         """
         raise NotImplementedError, "abstract function"
 
-    def end_output (self, linknumber=-1):
+    def end_output (self):
         """
         End of output, used for cleanup (eg output buffer flushing).
         """
