@@ -18,7 +18,6 @@
 A blacklist logger.
 """
 
-import sys
 import os
 
 import linkcheck
@@ -37,31 +36,25 @@ class BlacklistLogger (linkcheck.logger.Logger):
         Intialize with old blacklist data (if found, else not).
         """
         super(BlacklistLogger, self).__init__(**args)
+        self.init_fileoutput(args)
         self.blacklist = {}
         if args.get('fileoutput'):
-            self.fileoutput = True
             filename = args['filename']
             if os.path.exists(filename):
-                self.read_blacklist(file(filename, "r"))
-            self.fd = file(filename, "w")
-        elif args.has_key('fd'):
-            self.fd = args['fd']
-        else:
-            self.fileoutput = False
-            self.fd = sys.stdout
+                self.read_blacklist(filename)
 
     def comment (self, s, **args):
         """
-        Print nothing.
+        Write nothing.
         """
         pass
 
-    def new_url (self, url_data):
+    def log_url (self, url_data):
         """
         Put invalid url in blacklist, delete valid url from blacklist.
         """
         if not url_data.cached:
-            key = url_data.get_cache_key()
+            key = url_data.cache_url_key
             if key in self.blacklist:
                 if url_data.valid:
                     del self.blacklist[key]
@@ -71,16 +64,17 @@ class BlacklistLogger (linkcheck.logger.Logger):
                 if not url_data.valid:
                     self.blacklist[key] = 1
 
-    def end_output (self, linknumber=-1):
+    def end_output (self):
         """
         Write blacklist file.
         """
         self.write_blacklist()
 
-    def read_blacklist (self, fd):
+    def read_blacklist (self, filename):
         """
         Read a previously stored blacklist from file fd.
         """
+        fd = file(filename)
         for line in fd:
             line = line.rstrip()
             if line.startswith('#') or not line:
@@ -97,7 +91,6 @@ class BlacklistLogger (linkcheck.logger.Logger):
         for key, value in self.blacklist.items():
             self.fd.write("%d %s" % (value, key))
             self.fd.write(os.linesep)
-        if self.fileoutput:
-            self.fd.close()
+        self.close_fileoutput()
         # restore umask
         os.umask(oldmask)
