@@ -40,13 +40,13 @@ class DOTLogger (linkcheck.logger.Logger):
 
     def start_output (self):
         """
-        Print start of checking info as DOT comment.
+        Write start of checking info as DOT comment.
         """
         super(DOTLogger, self).start_output()
         if self.fd is None:
             return
         self.starttime = time.time()
-        if self.has_field("intro"):
+        if self.has_part("intro"):
             self.comment(_("created by %s at %s") % \
                          (linkcheck.configuration.AppName,
                           linkcheck.strformat.strtime(self.starttime)))
@@ -61,12 +61,12 @@ class DOTLogger (linkcheck.logger.Logger):
 
     def comment (self, s, **args):
         """
-        Print DOT comment.
+        Write DOT comment.
         """
         self.write(u"// ")
         self.writeln(s=s, **args)
 
-    def new_url (self, url_data):
+    def log_url (self, url_data):
         """
         Write one node and all possible edges.
         """
@@ -78,22 +78,22 @@ class DOTLogger (linkcheck.logger.Logger):
             self.nodes[node.url] = node
             self.nodeid += 1
             self.writeln(u"  %d [" % node.id)
-            if self.has_field("realurl"):
+            if self.has_part("realurl"):
                 self.writeln(u'    href="%s",' % dotquote(node.url))
-            if node.dltime >= 0 and self.has_field("dltime"):
+            if node.dltime >= 0 and self.has_part("dltime"):
                 self.writeln(u"    dltime=%d," % node.dltime)
-            if node.dlsize >= 0 and self.has_field("dlsize"):
+            if node.dlsize >= 0 and self.has_part("dlsize"):
                 self.writeln(u"    dlsize=%d," % node.dlsize)
-            if node.checktime and self.has_field("checktime"):
+            if node.checktime and self.has_part("checktime"):
                 self.writeln(u"    checktime=%d," % node.checktime)
-            if self.has_field("extern"):
-                self.writeln(u"    extern=%d," % (node.extern and 1 or 0))
+            if self.has_part("extern"):
+                self.writeln(u"    extern=%d," % (node.extern[0] and 1 or 0))
             self.writeln(u"  ];")
 
     def write_edges (self):
         """
         Write all edges we can find in the graph in a brute-force
-        manner. Better would be a mapping of parent urls.
+        manner. Better would be a mapping of parent URLs.
         """
         for node in self.nodes.values():
             if self.nodes.has_key(node.parent_url):
@@ -101,29 +101,26 @@ class DOTLogger (linkcheck.logger.Logger):
                 target = node.id
                 self.writeln(u"  %d -> %d [" % (source, target))
                 self.writeln(u'    label="%s",' % dotedge(node.name))
-                if self.has_field("result"):
+                if self.has_part("result"):
                     self.writeln(u"    valid=%d," % (node.valid and 1 or 0))
                 self.writeln(u"  ];")
         self.flush()
 
-    def end_output (self, linknumber=-1):
+    def end_output (self):
         """
-        Print end of checking info as DOT comment.
+        Write end of checking info as DOT comment.
         """
         if self.fd is None:
             return
         self.write_edges()
         self.writeln(u"}")
-        if self.has_field("outro"):
+        if self.has_part("outro"):
             self.stoptime = time.time()
             duration = self.stoptime - self.starttime
             self.comment(_("Stopped checking at %s (%s)")%\
                          (linkcheck.strformat.strtime(self.stoptime),
                           linkcheck.strformat.strduration(duration)))
-        self.flush()
-        if self.close_fd:
-            self.fd.close()
-        self.fd = None
+        self.close_fileoutput()
 
 
 def dotquote (s):
