@@ -195,6 +195,8 @@ class TestUrl (unittest.TestCase):
         self.assertEqual(url_norm(url), nurl)
         url = "http://example.com/a/../a/b"
         self.assertEqual(url_norm(url), nurl)
+        url = "http://example.com/../a/b"
+        self.assertEqual(url_norm(url), nurl)
 
     def test_norm_path_relative_dots (self):
         """
@@ -276,6 +278,9 @@ class TestUrl (unittest.TestCase):
         url = 'b/../../foo/'
         nurl = '../foo/'
         self.assertEqual(url_norm(url), nurl)
+        url = './foo'
+        nurl = 'foo'
+        self.assertEqual(url_norm(url), nurl)
 
     def test_norm_path_relative_slashes (self):
         """
@@ -290,7 +295,7 @@ class TestUrl (unittest.TestCase):
 
     def test_mail_url (self):
         """
-        Test mailto urls.
+        Test mailto URLs.
         """
         # no netloc and no path
         url = 'mailto:'
@@ -324,14 +329,27 @@ class TestUrl (unittest.TestCase):
         url = "news:$%&/`%"
         nurl = 'news:%A7%24%25%26/%B4%60%A7%25'
         self.assertEqual(url_norm(url), nurl)
+        url = "news:comp.infosystems.www.servers.unix"
+        nurl = url
+        self.assertEqual(url_norm(url), nurl)
         # javascript url
         url = "javascript:loadthis()"
         nurl = url
         self.assertEqual(url_norm(url), nurl)
+        # ldap url
+        url = "ldap://[2001:db8::7]/c=GB?objectClass?one"
+        nurl = "ldap://%5B2001:db8::7%5D/c=GB%3FobjectClass%3Fone"
+        self.assertEqual(url_norm(url), nurl)
+        url = "tel:+1-816-555-1212"
+        nurl = url
+        self.assertEqual(url_norm(url), nurl)
+        url = "urn:oasis:names:specification:docbook:dtd:xml:4.1.2"
+        nurl = "urn:oasis%3Anames%3Aspecification%3Adocbook%3Adtd%3Axml%3A4.1.2"
+        self.assertEqual(url_norm(url), nurl)
 
     def test_norm_with_auth (self):
         """
-        Test norming of urls with authentication tokens.
+        Test norming of URLs with authentication tokens.
         """
         url = "telnet://user@www.imadoofus.org"
         nurl = url
@@ -401,13 +419,11 @@ class TestUrl (unittest.TestCase):
         self.assert_(not linkcheck.url.url_is_absolute(url), repr(url))
 
     def test_nopathquote_chars (self):
-        # XXX use platform resource
-        if os.name != 'nt':
-            return
-        url = "file:///c|/msys/"
-        nurl = url
-        self.assertEqual(url_norm(url), nurl)
-        self.assert_(not linkcheck.url.url_needs_quoting(url))
+        if os.name == 'nt':
+            url = "file:///c|/msys/"
+            nurl = url
+            self.assertEqual(url_norm(url), nurl)
+            self.assert_(not linkcheck.url.url_needs_quoting(url))
 
     def test_idn_encoding (self):
         """
@@ -452,15 +468,16 @@ class TestUrl (unittest.TestCase):
         """
         u = "scid=kb;en-us;Q248840"
         self.assertEqual(linkcheck.url.url_parse_query(u), u)
+        u = "scid=kb;en-us;Q248840&b=c;hulla=bulla"
+        self.assertEqual(linkcheck.url.url_parse_query(u), u)
 
 
 def test_suite ():
     """
     Build and return a TestSuite.
     """
-    suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(TestUrl))
-    return suite
+    return unittest.makeSuite(TestUrl)
+
 
 if __name__ == '__main__':
     unittest.main()
