@@ -54,9 +54,8 @@ def test_suite ():
     """
     Build and return a TestSuite.
     """
-    suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(TestUrlBuild))
-    return suite
+    return unittest.makeSuite(TestUrlBuild)
+
 
 if __name__ == '__main__':
     unittest.main()
