@@ -124,7 +124,7 @@ class Threader (object):
         """
         self._reduce_threads()
 
-    def start_thread (self, func, args):
+    def start_thread (self, func, args, name=None):
         """
         Generate a new thread.
         """
@@ -133,7 +133,10 @@ class Threader (object):
             func(*args)
         else:
             self._acquire()
-            t = threading.Thread(None, func, None, args)
+            # thread names must be ASCII to avoid conversion problems
+            assert isinstance(name, str) or name is None, \
+                   "Invalid name %r" % name
+            t = threading.Thread(None, func, name, args)
             t.start()
             self.threads.append(t)
 
@@ -144,4 +147,3 @@ class Threader (object):
         return "Threader with %d threads (max %d)" % \
             (self.active_threads(), self.threads_max)
 
-
