@@ -23,6 +23,8 @@ import os
 import urlparse
 import urllib
 
+urlparse.uses_netloc.append('ldap')
+
 # constants defining url part indexes
 SCHEME = 0
 HOSTNAME = DOMAIN = 1
@@ -151,12 +153,22 @@ def parse_qsl (qs, keep_blank_values=0, strict_parsing=0):
         false (the default), errors are silently ignored. If true,
         errors raise a ValueError exception.
     @type strict_parsing: bool
-    @returns: a list, as G-d intended.
-    @rtype: list
+    @returns: list of triples (key, value, separator) where key and value
+      are the splitted CGI parameter and separator the used separator
+      for this CGI parameter which is either a semicolon or an ampersand
+    @rtype: list of triples
     """
-    pairs = [s for s in qs.split('&')]
+    pairs = []
+    name_value_amp = qs.split('&')
+    for name_value in name_value_amp:
+        if ';' in name_value:
+            pairs.extend([[x, ';'] for x in name_value.split(';')])
+            pairs[-1][1] = '&'
+        else:
+            pairs.append([name_value, '&'])
+    pairs[-1][1] = ''
     r = []
-    for name_value in pairs:
+    for name_value, sep in pairs:
         nv = name_value.split('=', 1)
         if len(nv) != 2:
             if strict_parsing:
@@ -172,7 +184,7 @@ def parse_qsl (qs, keep_blank_values=0, strict_parsing=0):
                 value = urllib.unquote(nv[1].replace('+', ' '))
             else:
                 value = nv[1]
-            r.append((name, value))
+            r.append((name, value, sep))
     return r
 
 
@@ -219,7 +231,9 @@ def url_fix_host (urlparts):
             userpass += "@"
         else:
             userpass = ""
-        host, port = urllib.splitnport(host)
+        newhost, port = urllib.splitnport(host)
+        if port is not None:
+            host = newhost
         # remove trailing dot
         if host.endswith("."):
             host = host[:-1]
@@ -260,17 +274,17 @@ def url_parse_query (query):
     else:
         append = ""
     l = []
-    for k, v in parse_qsl(query, True):
+    for k, v, sep in parse_qsl(query, True):
         k = urllib.quote(k, '/-:,;')
         if v:
             v = urllib.quote(v, '/-:,;')
-            l.append("%s=%s" % (k, v))
+            l.append("%s=%s%s" % (k, v, sep))
         elif v is None:
-            l.append(k)
+            l.append("%s%s" % (k, sep))
         else:
             # some sites do not work when the equal sign is missing
-            l.append("%s=" % k)
-    return '&'.join(l) + append
+            l.append("%s=%s" % (k, sep))
+    return ''.join(l) + append
 
 
 def url_norm (url):
@@ -291,18 +305,18 @@ def url_norm (url):
     is_idn = url_fix_host(urlparts)
     # query
     urlparts[3] = url_parse_query(urlparts[3])
-    if not urlparts[2]:
-        # empty path is allowed if url is non-hierarchical, or if both
-        # query and fragment are also empty
-        # note that in relative links, urlparts[0] might be empty
-        # in this case, do not make any assumptions
-        if urlparts[0] and \
-           urlparts[0] not in urlparse.non_hierarchical and \
-           (urlparts[3] or urlparts[4]):
-            urlparts[2] = '/'
-    else:
-        # fix redundant path parts
-        urlparts[2] = collapse_segments(urlparts[2])
+    is_hierarchical = urlparts[0] not in urlparse.non_hierarchical
+    if is_hierarchical:
+        # URL has a hierarchical path we should norm
+        if not urlparts[2]:
+            # Empty path is allowed if both query and fragment are also empty.
+            # Note that in relative links, urlparts[0] might be empty.
+            # In this case, do not make any assumptions.
+            if urlparts[0] and (urlparts[3] or urlparts[4]):
+                urlparts[2] = '/'
+        else:
+            # fix redundant path parts
+            urlparts[2] = collapse_segments(urlparts[2])
     # quote parts again
     urlparts[0] = urllib.quote(urlparts[0]) # scheme
     urlparts[1] = urllib.quote(urlparts[1], '@:') # host
@@ -315,6 +329,7 @@ def url_norm (url):
 
 
 _slashes_ro = re.compile(r"/+")
+_thisdir_ro = re.compile(r"^\./")
 _samedir_ro = re.compile(r"/\./|/\.$")
 _parentdir_ro = re.compile(r"^/(\.\./)+|/(?!\.\./)[^/]+/\.\.(/|$)")
 _relparentdir_ro = re.compile(r"^(?!\.\./)[^/]+/\.\.(/|$)")
@@ -328,13 +343,14 @@ def collapse_segments (path):
     # backslashes to be left alone, and finally quoted with '%5C')
     # But replacing has several positive effects:
     # - Prevents path attacks on Windows systems (using \.. parent refs)
-    # - Fixes bad urls where users used backslashes instead of slashes.
+    # - Fixes bad URLs where users used backslashes instead of slashes.
     #   This is a far more probable case than users having an intentional
     #   backslash in the path name.
     path = path.replace('\\', '/')
     # shrink multiple slashes to one slash
     path = _slashes_ro.sub("/", path)
     # collapse redundant path segments
+    path = _thisdir_ro.sub("", path)
     path = _samedir_ro.sub("/", path)
     # collapse parent path segments
     # note: here we exploit the fact that the replacements happen
@@ -367,14 +383,14 @@ def url_quote (url):
     urlparts[2] = urllib.quote(urlparts[2], '/=,') # path
     urlparts[3] = urllib.quote(urlparts[3], '&=,') # query
     l = []
-    for k, v in parse_qsl(urlparts[3], True): # query
+    for k, v, sep in parse_qsl(urlparts[3], True): # query
         k = urllib.quote(k, '/-:,')
         if v:
             v = urllib.quote(v, '/-:,')
-            l.append("%s=%s" % (k, v))
+            l.append("%s=%s%s" % (k, v, sep))
         else:
-            l.append(k)
-    urlparts[3] = '&'.join(l)
+            l.append("%s%s" % (k, sep))
+    urlparts[3] = ''.join(l)
     urlparts[4] = urllib.quote(urlparts[4]) # anchor
     return urlparse.urlunsplit(urlparts)
 
